@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2022, Peter Haag
+ *  Copyright (c) 2009-2023, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -222,6 +222,11 @@ static inline void ExpandRecord_v3(recordHeaderV3_t *v3Record, master_record_t *
                 output_record->server_nw_delay_usec = latency->usecServerNwDelay;
                 output_record->appl_latency_usec = latency->usecApplLatency;
             } break;
+            case EXsamplerInfoID: {
+                EXsamplerInfo_t *samplerInfo = (EXsamplerInfo_t *)((void *)elementHeader + sizeof(elementHeader_t));
+                output_record->exporterSampler = samplerInfo->exporter_sysid;
+                output_record->selectorID = samplerInfo->selectorID;
+            } break;
             case EXobservationID: {
                 EXobservation_t *observation = (EXobservation_t *)((void *)elementHeader + sizeof(elementHeader_t));
                 output_record->observationDomainID = observation->domainID;
@@ -429,7 +434,6 @@ static inline void ExpandRecord_v3(recordHeaderV3_t *v3Record, master_record_t *
         }
     }
     // map icmp type/code in it's own vars
-    output_record->icmp = output_record->dstPort;
     if (size != v3Record->size) {
         LogError("Record size info: '%u' not equal sum extensions: '%u'", v3Record->size, size);
         exit(255);
@@ -456,6 +460,11 @@ static inline void ExpandRecord_v3(recordHeaderV3_t *v3Record, master_record_t *
         output_record->numElements++;
     }
 
+#ifdef NSEL
+    if (output_record->msecFirst == 0) output_record->msecFirst = output_record->msecEvent;
+    if (output_record->msecLast == 0) output_record->msecFirst = output_record->msecEvent;
+#endif
+
 #ifdef DEVEL
     printf("Ordered extensions: %u\n", output_record->numElements);
     for (int i = 0; i <= output_record->numElements; i++) {
