@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2004-2022, Peter Haag
+ *  Copyright (c) 2004-2023, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -35,6 +35,7 @@
 #include <fcntl.h>
 #include <netinet/in.h>
 #include <pthread.h>
+#include <signal.h>
 #include <stdatomic.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -70,6 +71,8 @@
     (a)->flags = 0;      \
     (a)->type = DATA_BLOCK_TYPE_3;
 
+static const char *nf_creator[MAX_CREATOR] = {"unknown", "nfcapd", "nfpcapd", "sfcapd", "nfdump", "nfanon", "nfprofile", "geolookup", "ft2nfdump"};
+
 /* function prototypes */
 static HEAP_ALLOC(wrkmem, LZO1X_1_MEM_COMPRESS);
 
@@ -148,8 +151,8 @@ static int LZO_initialize(void) {
 }  // End of LZO_initialize
 
 static int LZ4_initialize(void) {
-    int lz4_buff_size = LZ4_compressBound(BUFFSIZE + sizeof(dataBlock_t));
-    if (lz4_buff_size > (2 * BUFFSIZE)) {
+    int lz4_buff_size = LZ4_compressBound(WRITE_BUFFSIZE);
+    if (lz4_buff_size > (BUFFSIZE - sizeof(dataBlock_t))) {
         LogError("LZ4_compressBound() error in %s line %d: Buffer too small", __FILE__, __LINE__);
         return 0;
     }
@@ -493,7 +496,7 @@ static nffile_t *NewFile(nffile_t *nffile) {
             return NULL;
         }
         for (int i = 0; i < (QueueSize + 4); i++) {
-            void *p = malloc(2 * BUFFSIZE);
+            void *p = malloc(BUFFSIZE);
             if (!p) {
                 LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
                 return 0;
@@ -693,7 +696,7 @@ nffile_t *OpenFile(char *filename, nffile_t *nffile) {
 
 }  // End of OpenFile
 
-nffile_t *OpenNewFile(char *filename, nffile_t *nffile, int compress, int encryption) {
+nffile_t *OpenNewFile(char *filename, nffile_t *nffile, int creator, int compress, int encryption) {
     int fd;
 
     if (encryption != NOT_ENCRYPTED) {
@@ -722,6 +725,7 @@ nffile_t *OpenNewFile(char *filename, nffile_t *nffile, int compress, int encryp
     nffile->file_header->created = time(NULL);
     nffile->file_header->compression = compress;
     nffile->file_header->encryption = encryption;
+    nffile->file_header->creator = creator;
 
     if (write(nffile->fd, (void *)nffile->file_header, sizeof(fileHeaderV2_t)) < sizeof(fileHeaderV2_t)) {
         LogError("write() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
@@ -941,6 +945,16 @@ void DisposeFile(nffile_t *nffile) {
     if (nffile->ident) free(nffile->ident);
     if (nffile->fileName) free(nffile->fileName);
 
+    for (size_t queueLen = queue_length(nffile->blockQueue); queueLen > 0; queueLen--) {
+        void *p = queue_pop(nffile->blockQueue);
+        if (p) free(p);
+    }
+
+    for (size_t queueLen = queue_length(nffile->processQueue); queueLen > 0; queueLen--) {
+        void *p = queue_pop(nffile->processQueue);
+        if (p) free(p);
+    }
+
     free(nffile);
 
 }  // End of DisposeFile
@@ -1020,7 +1034,7 @@ static dataBlock_t *nfread(nffile_t *nffile) {
 
     dbg_printf("ReadBlock - type: %u, size: %u, numRecords: %u, flags: %u\n", buff->type, buff->size, buff->NumRecords, buff->flags);
 
-    if (buff->size > BUFFSIZE || buff->size == 0 || buff->NumRecords == 0) {
+    if (buff->size > (BUFFSIZE - sizeof(dataBlock_t)) || buff->size == 0 || buff->NumRecords == 0) {
         // this is most likely a corrupt file
         LogError("Corrupt data file: Error buffer size %u", buff->size);
         queue_push(nffile->blockQueue, buff);
@@ -1080,6 +1094,11 @@ static dataBlock_t *nfread(nffile_t *nffile) {
 __attribute__((noreturn)) void *nfreader(void *arg) {
     nffile_t *nffile = (nffile_t *)arg;
 
+    /* Signal handling */
+    sigset_t set = {0};
+    sigfillset(&set);
+    pthread_sigmask(SIG_SETMASK, &set, NULL);
+
     int terminate = atomic_load(&nffile->terminate);
     int blockCount = 0;
     dataBlock_t *block_header = NULL;
@@ -1189,6 +1208,11 @@ __attribute__((noreturn)) void *nfwriter(void *arg) {
     nffile_t *nffile = (nffile_t *)arg;
 
     dbg_printf("nfwriter enter\n");
+    /* disable signal handling */
+    sigset_t set = {0};
+    sigfillset(&set);
+    pthread_sigmask(SIG_SETMASK, &set, NULL);
+
     dataBlock_t *block_header;
     while (1) {
         block_header = queue_pop(nffile->processQueue);
@@ -1300,7 +1324,7 @@ void ModifyCompressFile(int compress) {
 
         compression = nffile_r->file_header->compression;
         if (compression == compress) {
-            printf("File %s is already same compression methode\n", nffile_r->fileName);
+            printf("File %s is already same compression method\n", nffile_r->fileName);
             continue;
         }
 
@@ -1309,7 +1333,7 @@ void ModifyCompressFile(int compress) {
         outfile[MAXPATHLEN - 1] = '\0';
 
         // allocate output file
-        nffile_w = OpenNewFile(outfile, NULL, compress, NOT_ENCRYPTED);
+        nffile_w = OpenNewFile(outfile, NULL, FILE_CREATOR(nffile_r), compress, NOT_ENCRYPTED);
         if (!nffile_w) {
             DisposeFile(nffile_r);
             break;
@@ -1422,10 +1446,16 @@ int QueryFile(char *filename) {
             return 0;
         }
 
+        if (fileHeader.creator >= MAX_CREATOR) {
+            LogError("Creator ID %u out of range - set creator to unknown.", fileHeader.creator);
+            fileHeader.creator = CREATOR_UNKNOWN;
+        }
+
         struct tm *tbuff = localtime(&fileHeader.created);
         char t1[64];
         strftime(t1, 63, "%Y-%m-%d %H:%M:%S", tbuff);
         printf("Created    : %s\n", t1);
+        printf("Created by : %s\n", nf_creator[fileHeader.creator]);
         printf("nfdump     : %x\n", fileHeader.nfdversion);
         printf("encryption : %s\n", fileHeader.encryption ? "yes" : "no");
         printf("Appdx blks : %u\n", fileHeader.appendixBlocks);
@@ -1472,7 +1502,7 @@ int QueryFile(char *filename) {
             return 0;
         }
 
-        // Should never happen, as catched already in first check, but test it anyway ..
+        // Should never happen, as caught already in first check, but test it anyway ..
         if (ret == 0) {
             LogError("Unexpected eof. Expected %u blocks, counted %i", fileHeader.NumBlocks, i);
             close(fd);
@@ -1505,7 +1535,7 @@ int QueryFile(char *filename) {
                 return 0;
         }
 
-        if ((nffile->block_header->size) > BUFFSIZE) {
+        if ((nffile->block_header->size) > (BUFFSIZE - sizeof(dataBlock_t))) {
             LogError("Expected to seek beyond EOF! File corrupted");
             close(fd);
             return 0;
@@ -1552,7 +1582,7 @@ int QueryFile(char *filename) {
                 nffile->block_header = buff;
                 buff = b;
                 if (Uncompress_Block_LZO(buff, nffile->block_header, nffile->buff_size) < 0) {
-                    LogError("LZO decommpress failed");
+                    LogError("LZO decompress failed");
                     return 0;
                 }
             } break;
@@ -1561,7 +1591,7 @@ int QueryFile(char *filename) {
                 nffile->block_header = buff;
                 buff = b;
                 if (Uncompress_Block_LZ4(buff, nffile->block_header, nffile->buff_size) < 0) {
-                    LogError("LZ4 decommpress failed");
+                    LogError("LZ4 decompress failed");
                     return 0;
                 }
             } break;
@@ -1570,7 +1600,7 @@ int QueryFile(char *filename) {
                 nffile->block_header = buff;
                 buff = b;
                 if (Uncompress_Block_BZ2(buff, nffile->block_header, nffile->buff_size) < 0) {
-                    LogError("Bzip2 decommpress failed");
+                    LogError("Bzip2 decompress failed");
                     return 0;
                 }
             } break;
@@ -1593,7 +1623,7 @@ int QueryFile(char *filename) {
                 numRecords++;
             }
             if (blockSize != nffile->block_header->size) {
-                LogError("Error in block: %u, couted array size: %u != header size: %u\n", numBlocks, blockSize, nffile->block_header->size);
+                LogError("Error in block: %u, counted array size: %u != header size: %u\n", numBlocks, blockSize, nffile->block_header->size);
                 close(fd);
                 return 0;
             }
