@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2022, Peter Haag
+ *  Copyright (c) 2023, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -35,7 +35,6 @@
 #include <stdint.h>
 #include <sys/types.h>
 
-#include "config.h"
 #include "id.h"
 
 /*
@@ -44,7 +43,7 @@
  *
  * The goal of the V3 extension format:
  *  - simpler and more robust storage format.
- *  - independant of extension records. No specific ordering required.
+ *  - independent of extension records. No specific ordering required.
  *    makes it much easier for multi theaded data processing.
  *  - more flexible record handling, allows variable length records/
  *
@@ -122,14 +121,6 @@ typedef struct elementHeader_s {
 
 #define EXnull 0
 
-/*
-#define str(s) #s lose(s)
-#define foo 4
-str (foo)
-expands to `"foo" lose(4)'.
-#define FOO(A) ({int retval; retval = do_something(A); retval;})
-*/
-
 #define MemberSize(type, member) sizeof(((type *)0)->member)
 
 #define VARLENGTH 0xFFFF
@@ -152,6 +143,9 @@ typedef struct EXgenericFlow_s {
 #define OFFinBytes offsetof(EXgenericFlow_t, inBytes)
 #define SIZEinBytes MemberSize(EXgenericFlow_t, inBytes)
     uint16_t srcPort;
+#ifdef GOLANG
+    uint16_t dstPort;
+#else
     union {
         uint16_t dstPort;
         struct {
@@ -159,6 +153,7 @@ typedef struct EXgenericFlow_s {
             uint8_t icmpCode;
         };
     };
+#endif
 #define OFFsrcPort offsetof(EXgenericFlow_t, srcPort)
 #define SIZEsrcPort MemberSize(EXgenericFlow_t, srcPort)
 #define OFFdstPort offsetof(EXgenericFlow_t, dstPort)
@@ -230,7 +225,7 @@ typedef struct EXflowMisc_s {
     uint8_t flowEndReason;
 #define OFFflowEndReason offsetof(EXflowMisc_t, flowEndReason)
 #define SIZEflowEndReason MemberSize(EXflowMisc_t, flowEndReason)
-    // allign bytes
+    // align bytes
     uint8_t revTcpFlags;
     uint8_t fill;
 } EXflowMisc_t;
@@ -390,16 +385,11 @@ typedef struct EXlatency_s {
 
 typedef struct EXsamplerInfo_s {
 #define EXsamplerInfoID 18
-    int32_t id;               // id assigned by the exporting device
-    uint32_t interval;        // sampling interval
-    uint16_t mode;            // sampling mode
+    uint64_t selectorID;      // #302 id assigned by the exporting device
     uint16_t exporter_sysid;  // internal reference to exporter
-#define OFFsampID offsetof(EXsamplerInfo_t, id)
-#define SIZEsampID MemberSize(EXsamplerInfo_t, id)
-#define OFFsampInterval offsetof(EXsamplerInfo_t, interval)
-#define SIZEsampInterval MemberSize(EXsamplerInfo_t, interval)
-#define OFFsampMode offsetof(EXsamplerInfo_t, mode)
-#define SIZEsampMode MemberSize(EXsamplerInfo_t, mode)
+    uint16_t align;
+#define OFFsampID offsetof(EXsamplerInfo_t, selectorID)
+#define SIZEsampID MemberSize(EXsamplerInfo_t, selectorID)
 #define OFFsampExporter offsetof(EXsamplerInfo_t, exporter_sysid)
 #define SIZEsampExporter MemberSize(EXsamplerInfo_t, exporter_sysid)
 } EXsamplerInfo_t;
