@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2022, Peter Haag
+ *  Copyright (c) 2009-2023, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -33,9 +33,15 @@
 #include "config.h"
 #endif
 
+#ifndef SYSLOG_NAMES
+#define SYSLOG_NAMES 1
+#endif
+
 #include <arpa/inet.h>
 #include <errno.h>
 #include <limits.h>
+#include <stdarg.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -45,20 +51,10 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/types.h>
+#include <syslog.h>
 #include <time.h>
 #include <unistd.h>
 
-#ifndef SYSLOG_NAMES
-#define SYSLOG_NAMES 1
-#endif
-
-#include <stdarg.h>
-#include <syslog.h>
-
-#ifdef HAVE_STDINT_H
-#include <stdint.h>
-#endif
-
 #include "util.h"
 
 /* Global vars */
@@ -109,6 +105,10 @@ void xsleep(suseconds_t usec) {
 // Check cmd line argument length
 // exit on failure
 void CheckArgLen(char *arg, size_t len) {
+    if (arg == NULL) {
+        fprintf(stderr, "Input string error. Expected argument\n");
+        exit(EXIT_FAILURE);
+    }
     size_t i = 0;
     while (arg[i] != '\0' && i < len) i++;
     if (i > len) {
@@ -188,7 +188,7 @@ int CheckPath(char *path, unsigned type) {
     return ret == 2 ? 1 : 0;
 }  // End of CheckPath
 
-void EndLog() {
+void EndLog(void) {
     if (use_syslog) closelog();
 }  // End of CloseLog
 
@@ -197,7 +197,7 @@ int InitLog(int want_syslog, char *name, char *facility, int verbose_log) {
     char *logname;
 
 #ifdef DEVEL
-    verbose_log = 2;
+    verbose_log = 4;
 #endif
 
     verbose = verbose_log;
@@ -281,7 +281,6 @@ void LogInfo(char *format, ...) {
 void LogVerbose(char *format, ...) {
     va_list var_args;
     char string[512];
-
     if (verbose > 1) {
         va_start(var_args, format);
         vsnprintf(string, 511, format, var_args);
@@ -451,12 +450,12 @@ timeWindow_t *ScanTimeFrame(char *tstring) {
         errno = 0;
         long sec = strtol(tstring, &invalid, 10);
         if (sec == 0 && errno != 0) {
-            LogError("Invald time string %s: %s", tstring, strerror(errno));
+            LogError("Invalid time string %s: %s", tstring, strerror(errno));
             free(timeWindow);
             return NULL;
         }
         if (invalid && strlen(invalid) > 0) {
-            LogError("Invald time string %s at %c", tstring, *invalid);
+            LogError("Invalid time string %s at %c", tstring, *invalid);
             free(timeWindow);
             return NULL;
         }
