@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2021, Peter Haag
+ *  Copyright (c) 2021-2023, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -56,6 +56,12 @@
 #define kh_hash_func(key) (khint32_t)(key.key)
 #define kh_hash_equal(a, b) ((a.key) == (b.key))
 
+#define arrayElementSizeCheck(type)                                          \
+    if (arrayHeader->size != sizeof(type##_t)) {                             \
+        LogError("Size check failed for %s - rebuild nfdump geo DB", #type); \
+        return 0;                                                            \
+    }
+
 KHASH_INIT(localMap, locationKey_t, locationInfo_t, 1, kh_hash_func, kh_hash_equal)
 
 // ipV4Tree node compare function
@@ -500,38 +506,48 @@ int loadASV4tree(char *fileName) {
         *eol = '\0';
         // printf("%s\n", line);
         asV4Node_t asV4Node;
-        char *l = line;
-        char *field = NULL;
-        int i = 0;
-        while ((field = strsep(&l, ",")) != NULL) {
-            // printf("field: %s\n", field);
-            switch (i) {
-                case (0): {  // cidr
-                    char *cidr = strchr(field, '/');
-                    *cidr = '\0';
-                    uint32_t net, netBits, mask;
-                    int ret = inet_pton(PF_INET, field, &net);
-                    if (ret != 1) {
-                        LogError("Not an IPv4 network: %s\n", field);
-                        continue;
-                    }
-                    netBits = atoi(++cidr);
-                    mask = 0xffffffff << (32 - netBits);
-                    // printf("ip: 0x%x, bits: %u, mask: 0x%x\n", net, netBits, mask);
-                    asV4Node.network = ntohl(net);
-                    asV4Node.netmask = mask;
-                } break;
-                case 1:  // AS
-                    asV4Node.as = atoi(field);
-                    break;
-                case 2:  // org name
-                    strncpy(asV4Node.orgName, field, 64);
-                    asV4Node.orgName[63] = '\0';
-                    break;
-            }
-            i++;
-        }
+        char *field = line;
+        char *sep = NULL;
 
+        // extract cidr
+        sep = strchr(field, ',');
+        if (!sep) {
+            LogError("Parse cidr in ASv4 file: %s: failed", fileName);
+            return 0;
+        }
+        *sep++ = '\0';
+
+        // cidr
+        char *cidr = strchr(field, '/');
+        *cidr = '\0';
+        uint32_t net, netBits, mask;
+        int ret = inet_pton(PF_INET, field, &net);
+        if (ret != 1) {
+            LogError("Not an IPv4 network: %s\n", field);
+            continue;
+        }
+        netBits = atoi(++cidr);
+        mask = 0xffffffff << (32 - netBits);
+        // printf("ip: 0x%x, bits: %u, mask: 0x%x\n", net, netBits, mask);
+        asV4Node.network = ntohl(net);
+        asV4Node.netmask = mask;
+        field = sep;
+
+        // extract AS
+        sep = strchr(field, ',');
+        if (!sep) {
+            LogError("Parse AS in ASv4 file: %s: failed", fileName);
+            return 0;
+        }
+        *sep++ = '\0';
+        asV4Node.as = atoi(field);
+        field = sep;
+
+        // extract org name
+        strncpy(asV4Node.orgName, field, orgNameLength);
+        asV4Node.orgName[orgNameLength - 1] = '\0';
+
+        // insert node
         cnt++;
         asV4Node_t *node = kb_getp(asV4Tree, asV4Tree, &asV4Node);
         if (node) {
@@ -565,48 +581,57 @@ int loadASV6tree(char *fileName) {
         *eol = '\0';
         // printf("%s\n", line);
         asV6Node_t asV6Node;
-        char *l = line;
-        char *field = NULL;
-        int i = 0;
-        while ((field = strsep(&l, ",")) != NULL) {
-            // printf("field: %s\n", field);
-            switch (i) {
-                case (0): {
-                    char *cidr = strchr(field, '/');
-                    *cidr = '\0';
-                    uint32_t netBits;
-                    uint64_t net[2], mask[2];
-                    int ret = inet_pton(PF_INET6, field, net);
-                    if (ret != 1) {
-                        LogError("Not an IPv4 network: %s\n", field);
-                        continue;
-                    }
-                    netBits = atoi(++cidr);
+        char *field = line;
+        char *sep = NULL;
 
-                    if (netBits > 64) {
-                        mask[0] = 0xffffffffffffffffLL;
-                        mask[1] = 0xffffffffffffffffLL << (64 - netBits);
-                    } else {
-                        mask[0] = 0xffffffffffffffffLL << (64 - netBits);
-                        mask[1] = 0;
-                    }
-
-                    // printf("ip: 0x%x, bits: %u, mask: 0x%x\n", net, netBits, mask);
-                    asV6Node.network[0] = ntohll(net[0]);
-                    asV6Node.network[1] = ntohll(net[1]);
-                    asV6Node.netmask[0] = mask[0];
-                    asV6Node.netmask[1] = mask[1];
-                } break;
-                case 1:  // geoname_id
-                    asV6Node.as = atoi(field);
-                    break;
-                case 2:  // org name
-                    strncpy(asV6Node.orgName, field, 64);
-                    asV6Node.orgName[63] = '\0';
-                    break;
-            }
-            i++;
+        // extract cidr
+        sep = strchr(field, ',');
+        if (!sep) {
+            LogError("Parse cidr in ASv6 file: %s: failed", fileName);
+            return 0;
         }
+        *sep++ = '\0';
+
+        // cidr
+        char *cidr = strchr(field, '/');
+        *cidr = '\0';
+        uint32_t netBits;
+        uint64_t net[2], mask[2];
+        int ret = inet_pton(PF_INET6, field, net);
+        if (ret != 1) {
+            LogError("Not an IPv6 network: %s\n", field);
+            continue;
+        }
+        netBits = atoi(++cidr);
+
+        if (netBits > 64) {
+            mask[0] = 0xffffffffffffffffLL;
+            mask[1] = 0xffffffffffffffffLL << (64 - netBits);
+        } else {
+            mask[0] = 0xffffffffffffffffLL << (64 - netBits);
+            mask[1] = 0;
+        }
+
+        // printf("ip: 0x%x, bits: %u, mask: 0x%x\n", net, netBits, mask);
+        asV6Node.network[0] = ntohll(net[0]);
+        asV6Node.network[1] = ntohll(net[1]);
+        asV6Node.netmask[0] = mask[0];
+        asV6Node.netmask[1] = mask[1];
+        field = sep;
+
+        // extract AS
+        sep = strchr(field, ',');
+        if (!sep) {
+            LogError("Parse AS in ASv4 file: %s: failed", fileName);
+            return 0;
+        }
+        *sep++ = '\0';
+        asV6Node.as = atoi(field);
+        field = sep;
+
+        // extract org name
+        strncpy(asV6Node.orgName, field, orgNameLength);
+        asV6Node.orgName[orgNameLength - 1] = '\0';
 
         cnt++;
         asV6Node_t *node = kb_getp(asV6Tree, asV6Tree, &asV6Node);
@@ -628,7 +653,6 @@ static int StoreLocalMap(nffile_t *nffile) {
 
     void *outBuff = nffile->buff_ptr;
     size_t size = 0;
-    unsigned cnt = 0;
     for (khint_t k = kh_begin(localMap); k != kh_end(localMap); ++k) {  // traverse
         locationInfo_t locationInfo;
         if (kh_exist(localMap, k)) {  // test if a bucket contains data
@@ -650,7 +674,6 @@ static int StoreLocalMap(nffile_t *nffile) {
                 outBuff += sizeof(recordHeader_t);
             }
             memcpy(outBuff, &locationInfo, sizeof(locationInfo));
-            cnt++;
             outBuff += sizeof(locationInfo_t);
             size -= sizeof(locationInfo_t);
             nffile->block_header->size += sizeof(locationInfo_t);
@@ -810,7 +833,11 @@ static int StoreASV6tree(nffile_t *nffile) {
 }  // End of StoreASV6tree
 
 int SaveMaxMind(char *fileName) {
-    nffile_t *nffile = OpenNewFile(fileName, NULL, LZ4_COMPRESSED, NOT_ENCRYPTED);
+    nffile_t *nffile = OpenNewFile(fileName, NULL, CREATOR_LOOKUP, LZ4_COMPRESSED, NOT_ENCRYPTED);
+    if (!nffile) {
+        LogError("OpenNewFile() failed.");
+        return 0;
+    }
 
     StoreLocalMap(nffile);
     WriteBlock(nffile);
@@ -835,7 +862,6 @@ int LoadMaxMind(char *fileName) {
     if (!nffile) {
         return 0;
     }
-    unsigned cnt = 0;
     int done = 0;
     while (!done) {
         // get next data block from file
@@ -866,13 +892,14 @@ int LoadMaxMind(char *fileName) {
         size_t expected = (arrayHeader->size * nffile->block_header->NumRecords) + sizeof(record_header_t);
         if (expected != nffile->block_header->size) {
             LogError("Array size calculated: %u != expected: %u for element: %u", expected, nffile->block_header->size, arrayHeader->type);
-            continue;
+            return 0;
         }
 
         switch (arrayHeader->type) {
             case LocalInfoElementID: {
                 // khash_t(localMap) *localMap = mmHandle->localMap;
                 locationInfo_t *locationInfo = (locationInfo_t *)arrayElement;
+                arrayElementSizeCheck(locationInfo);
                 for (int i = 0; i < nffile->block_header->NumRecords; i++) {
                     int absent;
                     locationKey_t locationKey = {.key = locationInfo->localID};
@@ -883,12 +910,12 @@ int LoadMaxMind(char *fileName) {
                         kh_value(mmHandle->localMap, k) = *locationInfo;
                     }
                     locationInfo++;
-                    cnt++;
                 }
             } break;
             case IPV4treeElementID: {
                 kbtree_t(ipV4Tree) *ipV4Tree = mmHandle->ipV4Tree;
                 ipV4Node_t *ipV4Node = (ipV4Node_t *)arrayElement;
+                arrayElementSizeCheck(ipV4Node);
                 for (int i = 0; i < nffile->block_header->NumRecords; i++) {
                     ipV4Node_t *node = kb_getp(ipV4Tree, ipV4Tree, ipV4Node);
                     if (node) {
@@ -902,6 +929,7 @@ int LoadMaxMind(char *fileName) {
             case IPV6treeElementID: {
                 kbtree_t(ipV6Tree) *ipV6Tree = mmHandle->ipV6Tree;
                 ipV6Node_t *ipV6Node = (ipV6Node_t *)arrayElement;
+                arrayElementSizeCheck(ipV6Node);
                 for (int i = 0; i < nffile->block_header->NumRecords; i++) {
                     ipV6Node_t *node = kb_getp(ipV6Tree, ipV6Tree, ipV6Node);
                     if (node) {
@@ -916,6 +944,7 @@ int LoadMaxMind(char *fileName) {
             case ASV4treeElementID: {
                 kbtree_t(asV4Tree) *asV4Tree = mmHandle->asV4Tree;
                 asV4Node_t *asV4Node = (asV4Node_t *)arrayElement;
+                arrayElementSizeCheck(asV4Node);
                 for (int i = 0; i < nffile->block_header->NumRecords; i++) {
                     asV4Node_t *node = kb_getp(asV4Tree, asV4Tree, asV4Node);
                     if (node) {
@@ -929,6 +958,7 @@ int LoadMaxMind(char *fileName) {
             case ASV6treeElementID: {
                 kbtree_t(asV6Tree) *asV6Tree = mmHandle->asV6Tree;
                 asV6Node_t *asV6Node = (asV6Node_t *)arrayElement;
+                arrayElementSizeCheck(asV6Node);
                 for (int i = 0; i < nffile->block_header->NumRecords; i++) {
                     asV6Node_t *node = kb_getp(asV6Tree, asV6Tree, asV6Node);
                     if (node) {
@@ -1010,7 +1040,8 @@ void LookupLocation(uint64_t ip[2], char *location, size_t len) {
     }
 
     ipLocationInfo_t info = {0};
-    if (ip[0] == 0) {  // IPv4
+    uint64_t ipMask = 0xFFFFFFFF00000000LL;
+    if (ip[0] == 0 && ((ip[1] & ipMask) == 0)) {  // IPv4
         ipV4Node_t ipSearch = {.network = ip[1], .netmask = 0};
         ipV4Node_t *ipV4Node = kb_getp(ipV4Tree, mmHandle->ipV4Tree, &ipSearch);
         if (!ipV4Node) {
@@ -1045,7 +1076,8 @@ uint32_t LookupAS(uint64_t ip[2]) {
         return 0;
     }
 
-    if (ip[0] == 0) {  // IPv4
+    uint64_t ipMask = 0xFFFFFFFF00000000LL;
+    if (ip[0] == 0 && ((ip[1] & ipMask) == 0)) {  // IPv4
         asV4Node_t asSearch = {.network = ip[1], .netmask = 0};
         asV4Node_t *asV4Node = kb_getp(asV4Tree, mmHandle->asV4Tree, &asSearch);
         return asV4Node == NULL ? 0 : asV4Node->as;
@@ -1056,9 +1088,31 @@ uint32_t LookupAS(uint64_t ip[2]) {
         asV6Node_t *asV6Node = kb_getp(asV6Tree, mmHandle->asV6Tree, &asV6Search);
         return asV6Node == NULL ? 0 : asV6Node->as;
     }
+    // unreached
 
 }  // End of LookupAS
 
+const char *LookupASorg(uint64_t ip[2]) {
+    if (!mmHandle) {
+        return "";
+    }
+
+    uint64_t ipMask = 0xFFFFFFFF00000000LL;
+    if (ip[0] == 0 && ((ip[1] & ipMask) == 0)) {  // IPv4
+        asV4Node_t asSearch = {.network = ip[1], .netmask = 0};
+        asV4Node_t *asV4Node = kb_getp(asV4Tree, mmHandle->asV4Tree, &asSearch);
+        return asV4Node == NULL ? "" : asV4Node->orgName;
+    } else {  // IPv6
+        asV6Node_t asV6Search = {0};
+        asV6Search.network[0] = ip[0];
+        asV6Search.network[1] = ip[1];
+        asV6Node_t *asV6Node = kb_getp(asV6Tree, mmHandle->asV6Tree, &asV6Search);
+        return asV6Node == NULL ? "" : asV6Node->orgName;
+    }
+    // unreached
+
+}  // End of LookupASorg
+
 void LookupWhois(char *ip) {
     uint32_t as = 0;
     char *asOrg = NULL;
@@ -1077,15 +1131,26 @@ void LookupWhois(char *ip) {
         ipSearch.network[1] = ntohll(network[1]);
         asSearch.network[1] = ntohll(network[1]);
 
-        ipV6Node = kb_getp(ipV6Tree, mmHandle->ipV6Tree, &ipSearch);
-        if (ipV6Node) {
-            info = ipV6Node->info;
-        }
+        uint64_t testv4v6 = ipSearch.network[1] & 0xFFFFFFFF00000000LL;
+        if (ipSearch.network[0] == 0 && (testv4v6 == 0LL || testv4v6 == 0x0000ffff00000000LL)) {
+            uint32_t net = ipSearch.network[1];
+            asV4Node_t asSearch = {.network = net, .netmask = 0};
+            asV4Node_t *asV4Node = kb_getp(asV4Tree, mmHandle->asV4Tree, &asSearch);
+            if (asV4Node) {
+                as = asV4Node->as;
+                asOrg = asV4Node->orgName;
+            }
+        } else {
+            ipV6Node = kb_getp(ipV6Tree, mmHandle->ipV6Tree, &ipSearch);
+            if (ipV6Node) {
+                info = ipV6Node->info;
+            }
 
-        asV6Node_t *asV6Node = kb_getp(asV6Tree, mmHandle->asV6Tree, &asSearch);
-        if (asV6Node) {
-            as = asV6Node->as;
-            asOrg = asV6Node->orgName;
+            asV6Node_t *asV6Node = kb_getp(asV6Tree, mmHandle->asV6Tree, &asSearch);
+            if (asV6Node) {
+                as = asV6Node->as;
+                asOrg = asV6Node->orgName;
+            }
         }
 
     } else {
