@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2012-2022, Peter Haag
+ *  Copyright (c) 2012-2023, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -28,9 +28,12 @@
  *
  */
 
+#include "ipfix.h"
+
 #include <arpa/inet.h>
 #include <errno.h>
 #include <netinet/in.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -39,17 +42,11 @@
 #include <time.h>
 #include <unistd.h>
 
-#include "config.h"
-
-#ifdef HAVE_STDINT_H
-#include <stdint.h>
-#endif
-
 #include "bookkeeper.h"
 #include "collector.h"
+#include "config.h"
 #include "exporter.h"
 #include "fnf.h"
-#include "ipfix.h"
 #include "metric.h"
 #include "nbar.h"
 #include "nfdump.h"
@@ -63,6 +60,8 @@
 enum {
     STACK_NONE = 0,
     STACK_ICMP,
+    STACK_ICMPTYPE,
+    STACK_ICMPCODE,
     STACK_DSTPORT,
     STACK_SAMPLER,
     STACK_MSECFIRST,
@@ -76,7 +75,7 @@ enum {
 };
 
 /*
- * 	All Obervation Domains from all exporter are stored in a linked list
+ * 	All Observation Domains from all exporter are stored in a linked list
  *	which uniquely can identify each exporter/Observation Domain
  */
 typedef struct exporterDomain_s {
@@ -87,7 +86,7 @@ typedef struct exporterDomain_s {
 
     uint64_t packets;           // number of packets sent by this exporter
     uint64_t flows;             // number of flow records sent by this exporter
-    uint32_t sequence_failure;  // number of sequence failues
+    uint32_t sequence_failure;  // number of sequence failures
 
     // sampling information:
     // each flow source may have several sampler applied:
@@ -120,6 +119,8 @@ typedef struct exporterDomain_s {
 
 } exporterDomain_t;
 
+static int ExtensionsEnabled[MAXEXTENSIONS];
+
 static const struct ipfixTranslationMap_s {
     uint16_t id;  // IPFIX element id
 #define Stack_ONLY 0
@@ -132,9 +133,11 @@ static const struct ipfixTranslationMap_s {
 } ipfixTranslationMap[] = {
     {IPFIX_octetDeltaCount, SIZEinBytes, NumberCopy, EXgenericFlowID, OFFinBytes, STACK_NONE, "octetDeltaCount"},
     {IPFIX_packetDeltaCount, SIZEinPackets, NumberCopy, EXgenericFlowID, OFFinPackets, STACK_NONE, "packetDeltaCount"},
+    {IPFIX_initiatorPackets, SIZEinPackets, NumberCopy, EXgenericFlowID, OFFinPackets, STACK_NONE, "initiator packets"},
     {IPFIX_deltaFlowCount, SIZEflows, NumberCopy, EXcntFlowID, OFFflows, STACK_NONE, "deltaFlowCount"},
     {IPFIX_protocolIdentifier, SIZEproto, NumberCopy, EXgenericFlowID, OFFproto, STACK_NONE, "proto"},
     {IPFIX_ipClassOfService, SIZEsrcTos, NumberCopy, EXgenericFlowID, OFFsrcTos, STACK_NONE, "src tos"},
+    {IPFIX_forwardingStatus, SIZEfwdStatus, NumberCopy, EXgenericFlowID, OFFfwdStatus, STACK_NONE, "forwarding status"},
     {IPFIX_tcpControlBits, SIZEtcpFlags, NumberCopy, EXgenericFlowID, OFFtcpFlags, STACK_NONE, "TCP flags"},
     {IPFIX_SourceTransportPort, SIZEsrcPort, NumberCopy, EXgenericFlowID, OFFsrcPort, STACK_NONE, "src port"},
     {IPFIX_SourceIPv4Address, SIZEsrc4Addr, NumberCopy, EXipv4FlowID, OFFsrc4Addr, STACK_NONE, "src IPv4"},
@@ -148,17 +151,23 @@ static const struct ipfixTranslationMap_s {
     {IPFIX_bgpSourceAsNumber, SIZEsrcAS, NumberCopy, EXasRoutingID, OFFsrcAS, STACK_NONE, "src AS"},
     {IPFIX_bgpDestinationAsNumber, SIZEdstAS, NumberCopy, EXasRoutingID, OFFdstAS, STACK_NONE, "dst AS"},
     {IPFIX_bgpNextHopIPv4Address, SIZEbgp4NextIP, NumberCopy, EXbgpNextHopV4ID, OFFbgp4NextIP, STACK_NONE, "IPv4 bgp next hop"},
-    {IPFIX_flowEndSysUpTime, Stack_ONLY, NumberCopy, EXnull, 0, STACK_MSECFIRST, "msec last SysupTime"},
-    {IPFIX_flowStartSysUpTime, Stack_ONLY, NumberCopy, EXnull, 0, STACK_MSECLAST, "msec first SysupTime"},
+    {IPFIX_flowEndSysUpTime, Stack_ONLY, NumberCopy, EXnull, 0, STACK_MSECLAST, "msec last SysupTime"},
+    {IPFIX_flowStartSysUpTime, Stack_ONLY, NumberCopy, EXnull, 0, STACK_MSECFIRST, "msec first SysupTime"},
     {IPFIX_SystemInitTimeMiliseconds, Stack_ONLY, NumberCopy, EXnull, 0, STACK_SYSUPTIME, "SysupTime msec"},
     {IPFIX_postOctetDeltaCount, SIZEoutBytes, NumberCopy, EXcntFlowID, OFFoutBytes, STACK_NONE, "output bytes delta counter"},
     {IPFIX_postPacketDeltaCount, SIZEoutPackets, NumberCopy, EXcntFlowID, OFFoutPackets, STACK_NONE, "output packet delta counter"},
+    {IPFIX_responderPackets, SIZEoutPackets, NumberCopy, EXcntFlowID, OFFoutPackets, STACK_NONE, "responder packets"},
+    {IPFIX_newconnections, SIZEflows, NumberCopy, EXcntFlowID, OFFflows, STACK_NONE, "connections"},
     {IPFIX_SourceIPv6Address, SIZEsrc6Addr, NumberCopy, EXipv6FlowID, OFFsrc6Addr, STACK_NONE, "IPv6 src addr"},
     {IPFIX_DestinationIPv6Address, SIZEdst6Addr, NumberCopy, EXipv6FlowID, OFFdst6Addr, STACK_NONE, "IPv6 dst addr"},
     {IPFIX_SourceIPv6PrefixLength, SIZEsrcMask, NumberCopy, EXflowMiscID, OFFsrcMask, STACK_NONE, "src mask bits"},
     {IPFIX_DestinationIPv6PrefixLength, SIZEdstMask, NumberCopy, EXflowMiscID, OFFdstMask, STACK_NONE, "dst mask bits"},
     {IPFIX_icmpTypeCodeIPv4, SIZEdstPort, NumberCopy, EXgenericFlowID, OFFdstPort, STACK_ICMP, "icmp v4 type/code"},
     {IPFIX_icmpTypeCodeIPv6, SIZEdstPort, NumberCopy, EXgenericFlowID, OFFdstPort, STACK_ICMP, "icmp v6 type/code"},
+    {IPFIX_icmpTypeV4, SIZEicmpCode, Stack_ONLY, EXgenericFlowID, OFFicmpType, STACK_ICMPTYPE, "icmp v4 type"},
+    {IPFIX_icmpCodeV4, SIZEicmpType, Stack_ONLY, EXgenericFlowID, OFFicmpCode, STACK_ICMPCODE, "icmp v4 code"},
+    {IPFIX_icmpTypeV6, SIZEicmpCode, Stack_ONLY, EXgenericFlowID, OFFicmpType, STACK_ICMPTYPE, "icmp v6 type"},
+    {IPFIX_icmpCodeV6, SIZEicmpType, Stack_ONLY, EXgenericFlowID, OFFicmpCode, STACK_ICMPCODE, "icmp v6 code"},
     {IPFIX_postIpClassOfService, SIZEdstTos, NumberCopy, EXflowMiscID, OFFdstTos, STACK_NONE, "post IP class of Service"},
     {IPFIX_SourceMacAddress, SIZEinSrcMac, NumberCopy, EXmacAddrID, OFFinSrcMac, STACK_NONE, "in src MAC addr"},
     {IPFIX_postDestinationMacAddress, SIZEoutDstMac, NumberCopy, EXmacAddrID, OFFoutDstMac, STACK_NONE, "out dst MAC addr"},
@@ -195,12 +204,12 @@ static const struct ipfixTranslationMap_s {
     {IPFIX_postPacketTotalCount, SIZEoutPackets, NumberCopy, EXcntFlowID, OFFoutPackets, STACK_NONE, "output packetTotalCount"},
     {IPFIX_engineType, Stack_ONLY, NumberCopy, EXnull, 0, STACK_ENGINETYPE, "engine type"},
     {IPFIX_engineId, Stack_ONLY, NumberCopy, EXnull, 0, STACK_ENGINEID, "engine ID"},
-    {NBAR_APPLICATION_ID, SIZEnbarAppID, NumberCopy, EXnbarAppID, OFFnbarAppID, STACK_NONE, "nbar application ID"},
+    {NBAR_APPLICATION_ID, SIZEnbarAppID, ByteCopy, EXnbarAppID, OFFnbarAppID, STACK_NONE, "nbar application ID"},
     {IPFIX_observationDomainId, SIZEdomainID, NumberCopy, EXobservationID, OFFdomainID, STACK_NONE, "observation domainID"},
     {IPFIX_observationPointId, SIZEpointID, NumberCopy, EXobservationID, OFFpointID, STACK_NONE, "observation pointID"},
     // sampling
-    {IPFIX_samplerId, Stack_ONLY, NumberCopy, EXnull, 0, STACK_SAMPLER, "sampler ID"},
-    {IPFIX_selectorId, Stack_ONLY, NumberCopy, EXnull, 0, STACK_SAMPLER, "sampler ID"},
+    {IPFIX_samplerId, SIZEsampID, NumberCopy, EXsamplerInfoID, OFFsampID, STACK_SAMPLER, "sampler ID"},
+    {IPFIX_selectorId, SIZEsampID, NumberCopy, EXsamplerInfoID, OFFsampID, STACK_SAMPLER, "sampler ID"},
     {IPFIX_INGRESS_VRFID, SIZEingressVrf, NumberCopy, EXvrfID, OFFingressVrf, STACK_NONE, "ingress VRF ID"},
     {IPFIX_EGRESS_VRFID, SIZEegressVrf, NumberCopy, EXvrfID, OFFegressVrf, STACK_NONE, "egress VRF ID"},
     // NAT
@@ -262,20 +271,52 @@ static int LookupElement(uint16_t type, uint32_t EnterpriseNumber);
 #include "inline.c"
 #include "nffile_inline.c"
 
-int Init_IPFIX(int verbose, int32_t sampling) {
+int Init_IPFIX(int verbose, int32_t sampling, char *extensionList) {
     printRecord = verbose > 2;
 
     defaultSampling = sampling;
 
-    int i;
-    for (i = 0; ipfixTranslationMap[i].name != NULL; i++) {
+    if (extensionList) {
+        // Disable all extensions
+        for (int i = 0; i < MAXEXTENSIONS; i++) {
+            ExtensionsEnabled[i] = 0;
+        }
+
+        // get enabled extensions from string
+        int extID = ScanExtension(extensionList);
+        while (extID > 0) {
+            dbg_printf("Enable extension %d\n", extID);
+            ExtensionsEnabled[extID] = 1;
+            extID = ScanExtension(NULL);
+        }
+
+        if (extID == -1) {
+            LogError("Failed to scan extension list.");
+            return 0;
+        }
+
+        // make sure extension 1 is enabled
+        ExtensionsEnabled[1] = 1;
+
+    } else {
+        // Enable all extensions
+        dbg_printf("Enable all extensions\n");
+        for (int i = 0; i < MAXEXTENSIONS; i++) {
+            ExtensionsEnabled[i] = 1;
+        }
+    }
+
+    int tagsEnabled = 0;
+    for (int i = 0; ipfixTranslationMap[i].name != NULL; i++) {
+        int extID = ipfixTranslationMap[i].extensionID;
+        if (ExtensionsEnabled[extID]) tagsEnabled++;
     }
 
     if (sampling < 0) {
-        LogInfo("Init IPFIX: Max number of ipfix tags: %u, overwrite sampling: %d", i, -defaultSampling);
+        LogInfo("Init IPFIX: Max number of ipfix tags enabled: %u, overwrite sampling: %d", tagsEnabled, -defaultSampling);
         dbg_printf("Initv9: Overwrite sampling: %d\n", -defaultSampling);
     } else {
-        LogInfo("Init IPFIX: Max number of ipfix tags: %u, default sampling: %d", i, defaultSampling);
+        LogInfo("Init IPFIX: Max number of ipfix tags enabled: %u, default sampling: %d", tagsEnabled, defaultSampling);
         dbg_printf("Initv9: Default sampling: %d\n", defaultSampling);
     }
 
@@ -287,6 +328,30 @@ static int LookupElement(uint16_t type, uint32_t EnterpriseNumber) {
     switch (EnterpriseNumber) {
         case 0:  // no Enterprise value
             break;
+        case 9:  // CiscoSystem
+            switch (type) {
+                case 12236:  // client ipv4 address
+                    dbg_printf(" CISCO enterprise client IP type: %u\n", type);
+                    type = IPFIX_SourceIPv4Address;
+                    break;
+                case 12237:  // server ipv4 address
+                    dbg_printf(" CISCO enterprise server IP type: %u\n", type);
+                    type = IPFIX_DestinationIPv4Address;
+                    break;
+                case 12241:  // server transport port
+                    dbg_printf(" CISCO enterprise server port type: %u\n", type);
+                    type = IPFIX_DestinationTransportPort;
+                    break;
+                case 8337:  //  server counter bytes network
+                    dbg_printf(" CISCO enterprise server bytes type: %u\n", type);
+                    type = IPFIX_octetDeltaCount;
+                    break;
+                case 8338:  //  client counter bytes network
+                    dbg_printf(" CISCO enterprise client bytes type: %u\n", type);
+                    type = IPFIX_postOctetDeltaCount;
+                    break;
+            }
+            break;
         case 6871:  // yaf CERT Coordination Centre
             // map yaf types here
             switch (type) {
@@ -317,7 +382,15 @@ static int LookupElement(uint16_t type, uint32_t EnterpriseNumber) {
 
     int i = 0;
     while (ipfixTranslationMap[i].name != NULL) {
-        if (ipfixTranslationMap[i].id == type) return i;
+        if (ipfixTranslationMap[i].id == type) {
+            int extID = ipfixTranslationMap[i].extensionID;
+            if (ExtensionsEnabled[extID]) {
+                return i;
+            } else {
+                dbg_printf("Extension %d not enabled\n", extID);
+                return -1;
+            }
+        }
         i++;
     }
 
@@ -567,6 +640,7 @@ static void removeTemplate(exporterDomain_t *exporter, uint16_t id) {
     if (template->type == DATA_TEMPLATE) {
         dataTemplate_t *dataTemplate = (dataTemplate_t *)template->data;
         ClearSequencer(&(dataTemplate->sequencer));
+        if (dataTemplate->extensionList) free(dataTemplate->extensionList);
     }
     free(template->data);
     free(template);
@@ -588,6 +662,7 @@ static void removeAllTemplates(exporterDomain_t *exporter) {
         if (template->type == DATA_TEMPLATE) {
             dataTemplate_t *dataTemplate = (dataTemplate_t *)template->data;
             ClearSequencer(&(dataTemplate->sequencer));
+            if (dataTemplate->extensionList) free(dataTemplate->extensionList);
         }
         free(template->data);
         free(template);
@@ -664,7 +739,7 @@ static void Process_ipfix_templates(exporterDomain_t *exporter, void *flowset_he
 static inline int SetSequence(sequence_t *sequenceTable, uint32_t numSequences, uint16_t Type, uint16_t Length, uint16_t EnterpriseNumber) {
     int found = 0;
     int index = LookupElement(Type, EnterpriseNumber);
-    if (index < 0) {  // not found - enter skip seqence
+    if (index < 0) {  // not found - enter skip sequence
         if ((EnterpriseNumber == 0) && (Type == IPFIX_subTemplateList || Type == IPFIX_subTemplateMultiList)) {
             sequenceTable[numSequences].inputType = Type;
             dbg_printf(" Add sequence for sub template type: %u, enterprise: %u, length: %u\n", Type, EnterpriseNumber, Length);
@@ -813,7 +888,7 @@ static void Process_ipfix_template_add(exporterDomain_t *exporter, void *DataPtr
         size_left -= size_required;
         DataPtr = DataPtr + size_required + 4;  // +4 for header
         if (size_left < 4) {
-            // pading
+            // padding
             dbg_printf("Skip %u bytes padding\n", size_left);
             size_left = 0;
         }
@@ -848,7 +923,7 @@ static void Process_ipfix_template_withdraw(exporterDomain_t *exporter, void *Da
 
         DataPtr = DataPtr + 4;
         if (size_left < 4) {
-            // pading
+            // padding
             dbg_printf("Skip %u bytes padding\n", size_left);
             size_left = 0;
         }
@@ -860,7 +935,7 @@ static void Process_ipfix_option_templates(exporterDomain_t *exporter, void *opt
     uint8_t *option_template;
     uint32_t size_left, size_required;
     // uint32_t nr_scopes, nr_options;
-    uint16_t tableID, field_count, scope_field_count, offset;
+    uint16_t tableID, field_count, scope_field_count;
 
     size_left = GET_FLOWSET_LENGTH(option_template_flowset) - 4;  // -4 for flowset header -> id and length
     if (size_left < 6) {
@@ -902,37 +977,6 @@ static void Process_ipfix_option_templates(exporterDomain_t *exporter, void *opt
         return;
     }
 
-    int i;
-    offset = 0;
-    for (i = 0; i < scope_field_count; i++) {
-        uint16_t id, length;
-        int Enterprise;
-
-        if (size_left && size_left < 4) {
-            LogError("Process_ipfix [%u] Template size error at %s line %u", exporter->info.id, __FILE__, __LINE__, strerror(errno));
-            return;
-        }
-        id = Get_val16(option_template);
-        option_template += 2;
-        length = Get_val16(option_template);
-        option_template += 2;
-        size_left -= 4;
-        Enterprise = id & 0x8000 ? 1 : 0;
-        if (Enterprise) {
-            size_required += 4;
-            if (size_left < 4) {
-                dbg_printf("option template length error: size left %u too small\n", size_left);
-                return;
-            }
-            option_template += 4;
-            size_left -= 4;
-            dbg_printf(" [%i] Enterprise: 1, scope id: %u, scope length %u enterprise value: %u\n", i, id, length, Get_val32(option_template));
-        } else {
-            dbg_printf(" [%i] Enterprise: 0, scope id: %u, scope length %u\n", i, id, length);
-        }
-        offset += length;
-    }
-
     removeTemplate(exporter, tableID);
     optionTemplate_t *optionTemplate = (optionTemplate_t *)calloc(1, sizeof(optionTemplate_t));
     if (!optionTemplate) {
@@ -940,14 +984,14 @@ static void Process_ipfix_option_templates(exporterDomain_t *exporter, void *opt
         return;
     }
 
-    uint16_t scopeSize = offset;
-
     struct samplerOption_s *samplerOption = &(optionTemplate->samplerOption);
     struct nbarOptionList_s *nbarOption = &(optionTemplate->nbarOption);
     struct nameOptionList_s *ifnameOptionList = &(optionTemplate->ifnameOption);
     struct nameOptionList_s *vrfnameOptionList = &(optionTemplate->vrfnameOption);
 
-    for (; i < field_count; i++) {
+    uint16_t scopeSize = 0;
+    uint16_t offset = 0;
+    for (int i = 0; i < field_count; i++) {
         uint32_t enterprise_value;
         uint16_t type, length;
         int Enterprise;
@@ -959,6 +1003,13 @@ static void Process_ipfix_option_templates(exporterDomain_t *exporter, void *opt
         length = Get_val16(option_template);
         option_template += 2;
         size_left -= 4;
+        if (i < scope_field_count) {
+            scopeSize += length;
+            dbg_printf("Scope field Type: %u, offset: %u, length %u\n", type, offset, length);
+        } else {
+            dbg_printf("Option field Type: %u, offset: %u, length %u\n", type, offset, length);
+        }
+
         Enterprise = type & 0x8000 ? 1 : 0;
         if (Enterprise) {
             size_required += 4;
@@ -977,7 +1028,7 @@ static void Process_ipfix_option_templates(exporterDomain_t *exporter, void *opt
         }
 
         switch (type) {
-            // general sampling
+            // Old std sampling tags
             case IPFIX_samplingInterval:  // #34
                 samplerOption->spaceInterval.length = length;
                 samplerOption->spaceInterval.offset = offset;
@@ -991,9 +1042,10 @@ static void Process_ipfix_option_templates(exporterDomain_t *exporter, void *opt
                 dbg_printf(" Sampling #35 found\n");
                 break;
 
-            // individual samplers
+            // New std sampling, individual sammplers (sampling ID)
+            // Map old individual samplers
             case IPFIX_samplerId:  // #48 deprecated - fall through
-                dbg_printf(" Sampling #48 found\n");
+                dbg_printf(" Sampling #48 map to #302\n");
             case IPFIX_selectorId:  // #302
                 samplerOption->id.length = length;
                 samplerOption->id.offset = offset;
@@ -1084,8 +1136,9 @@ static void Process_ipfix_option_templates(exporterDomain_t *exporter, void *opt
         }
         offset += length;
     }
+    optionTemplate->optionSize = offset;
 
-    dbg_printf("\n[%u] Option flags: %llx\n", exporter->info.id, optionTemplate->flags);
+    dbg_printf("\n[%u] Option size: %llu, flags: %llx\n", exporter->info.id, optionTemplate->optionSize, optionTemplate->flags);
     if (optionTemplate->flags) {
         // if it exitsts - remove old template on exporter with same ID
         templateList_t *template = newTemplate(exporter, tableID);
@@ -1095,11 +1148,19 @@ static void Process_ipfix_option_templates(exporterDomain_t *exporter, void *opt
         }
         template->data = optionTemplate;
 
-        if ((optionTemplate->flags & SAMPLERMASK) == SAMPLER306) {
-            dbg_printf("[%u] Sampler information found\n", exporter->info.id);
+        if ((optionTemplate->flags & SAMPLERFLAGS) == SAMPLERFLAGS) {
+            dbg_printf("[%u] New Sampler information found\n", exporter->info.id);
             SetFlag(template->type, SAMPLER_TEMPLATE);
-        } else if ((optionTemplate->flags & STDMASK) == STDSAMPLING34) {
-            dbg_printf("[%u] Std sampling information found\n", exporter->info.id);
+        } else if ((optionTemplate->flags & SAMPLERSTDFLAGS) == SAMPLERSTDFLAGS) {
+            dbg_printf("[%u] New std sampling information found\n", exporter->info.id);
+            SetFlag(template->type, SAMPLER_TEMPLATE);
+        } else if ((optionTemplate->flags & STDMASK) == STDFLAGS) {
+            dbg_printf("[%u] Old std sampling information found\n", exporter->info.id);
+            SetFlag(template->type, SAMPLER_TEMPLATE);
+        } else if ((optionTemplate->flags & STDSAMPLING34) == STDSAMPLING34) {
+            dbg_printf("[%u] Old std sampling information found - missing algorithm\n", exporter->info.id);
+            samplerOption->algorithm.length = 0;
+            samplerOption->algorithm.offset = 0;
             SetFlag(template->type, SAMPLER_TEMPLATE);
         } else {
             dbg_printf("[%u] No Sampling information found\n", exporter->info.id);
@@ -1107,9 +1168,10 @@ static void Process_ipfix_option_templates(exporterDomain_t *exporter, void *opt
 
         if (TestFlag(optionTemplate->flags, NBAROPTIONS)) {
             dbg_printf("[%u] found nbar options\n", exporter->info.id);
-            dbg_printf("[%u] id   length: %u\n", exporter->info.id, nbarOption->id.length);
-            dbg_printf("[%u] name length: %u\n", exporter->info.id, nbarOption->name.length);
-            dbg_printf("[%u] desc length: %u\n", exporter->info.id, nbarOption->desc.length);
+            dbg_printf("[%u] id   length: %u, offset: %u\n", exporter->info.id, nbarOption->id.length, nbarOption->id.offset);
+            dbg_printf("[%u] name length: %u, offset: %u\n", exporter->info.id, nbarOption->name.length, nbarOption->name.offset);
+            dbg_printf("[%u] desc length: %u, offset: %u\n", exporter->info.id, nbarOption->desc.length, nbarOption->desc.offset);
+            optionTemplate->nbarOption.scopeSize = scopeSize;
             SetFlag(template->type, NBAR_TEMPLATE);
         } else {
             dbg_printf("[%u] No nbar information found\n", exporter->info.id);
@@ -1165,9 +1227,9 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
     // reserve space in output stream for EXipReceivedVx
     uint32_t receivedSize = 0;
     if (fs->sa_family == PF_INET6)
-        receivedSize = EXipReceivedV6Size;
+        receivedSize = ExtensionsEnabled[EXipReceivedV6ID] ? EXipReceivedV6Size : 0;
     else
-        receivedSize = EXipReceivedV4Size;
+        receivedSize = ExtensionsEnabled[EXipReceivedV4ID] ? EXipReceivedV4Size : 0;
 
     while (size_left > 0) {
         void *outBuff;
@@ -1182,7 +1244,7 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
 
         int buffAvail = CheckBufferSpace(fs->nffile, sizeof(recordHeaderV3_t) + outRecordSize + receivedSize);
         if (buffAvail == 0) {
-            // this should really never occur, because the buffer gets flushed ealier
+            // this should really never occur, because the buffer gets flushed earlier
             LogError("Process_ipfix: output buffer size error. Skip ipfix record processing");
             dbg_printf("Process_ipfix: output buffer size error. Skip ipfix record processing");
             return;
@@ -1223,7 +1285,7 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
                 LogInfo("Process ipfix: Sequencer run - resize output buffer");
                 buffAvail = CheckBufferSpace(fs->nffile, buffAvail + 1);
                 if (buffAvail == 0) {
-                    // this should really never occur, because the buffer gets flushed ealier
+                    // this should really never occur, because the buffer gets flushed earlier
                     LogError("Process_ipfix: output buffer size error. Skip ipfix record processing");
                     dbg_printf("Process_ipfix: output buffer size error. Skip ipfix record processing");
                     return;
@@ -1242,14 +1304,22 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
 
         // add router IP
         if (fs->sa_family == PF_INET6) {
-            PushExtension(recordHeaderV3, EXipReceivedV6, ipReceivedV6);
-            ipReceivedV6->ip[0] = fs->ip.V6[0];
-            ipReceivedV6->ip[1] = fs->ip.V6[1];
-            dbg_printf("Add IPv6 route IP extension\n");
+            if (ExtensionsEnabled[EXipReceivedV6ID]) {
+                PushExtension(recordHeaderV3, EXipReceivedV6, ipReceivedV6);
+                ipReceivedV6->ip[0] = fs->ip.V6[0];
+                ipReceivedV6->ip[1] = fs->ip.V6[1];
+                dbg_printf("Add IPv6 route IP extension\n");
+            } else {
+                dbg_printf("IPv6 route IP extension not enabled\n");
+            }
         } else {
-            PushExtension(recordHeaderV3, EXipReceivedV4, ipReceivedV4);
-            ipReceivedV4->ip = fs->ip.V4;
-            dbg_printf("Add IPv4 route IP extension\n");
+            if (ExtensionsEnabled[EXipReceivedV4ID]) {
+                PushExtension(recordHeaderV3, EXipReceivedV4, ipReceivedV4);
+                ipReceivedV4->ip = fs->ip.V4;
+                dbg_printf("Add IPv4 route IP extension\n");
+            } else {
+                dbg_printf("IPv4 route IP extension not enabled\n");
+            }
         }
 
         dbg_printf("Record: %u elements, size: %u\n", recordHeaderV3->numElements, recordHeaderV3->size);
@@ -1279,6 +1349,11 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
             sampler = sampler->next;
         }
 
+        EXsamplerInfo_t *samplerInfo = (EXsamplerInfo_t *)sequencer->offsetCache[EXsamplerInfoID];
+        if (samplerInfo) {
+            samplerInfo->exporter_sysid = exporter->info.sysid;
+        }
+
         if (overwriteSampler) {
             // hard overwrite sampling
             packetInterval = overwriteSampler->record.packetInterval;
@@ -1358,6 +1433,8 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
                     }
                     if (stack[STACK_ICMP] != 0) {
                         genericFlow->dstPort = stack[STACK_ICMP];
+                    } else if (stack[STACK_ICMPTYPE] != 0 || stack[STACK_ICMPCODE] != 0) {
+                        genericFlow->dstPort = (stack[STACK_ICMPTYPE] << 8) + stack[STACK_ICMPCODE];
                     }
                     break;
                 case IPPROTO_TCP:
@@ -1440,7 +1517,7 @@ static inline void Process_ipfix_sampler_option_data(exporterDomain_t *exporter,
     optionTemplate_t *optionTemplate = (optionTemplate_t *)template->data;
     struct samplerOption_s *samplerOption = &(optionTemplate->samplerOption);
 
-    if ((optionTemplate->flags & SAMPLERMASK) != 0) {
+    if ((optionTemplate->flags & SAMPLERSTDFLAGS) != 0) {
         sampler_record_t sampler_record = {0};
 
         if (CHECK_OPTION_DATA(size_left, samplerOption->id)) {
@@ -1465,9 +1542,16 @@ static inline void Process_ipfix_sampler_option_data(exporterDomain_t *exporter,
                 LogError("Process_ipfix_option: Zero sampling interval -> sampling == 1", __FILE__, __LINE__);
             }
         }
+
         dbg_printf("Extracted Sampler data:\n");
-        dbg_printf("ID : %lld, algorithm : %u, packet interval: %u, packet space: %u\n", sampler_record.id, sampler_record.algorithm,
-                   sampler_record.packetInterval, sampler_record.spaceInterval);
+        if (sampler_record.id == 0) {
+            sampler_record.id = SAMPLER_GENERIC;
+            dbg_printf("New std sampler: algorithm : %u, packet interval: %u, packet space: %u\n", sampler_record.algorithm,
+                       sampler_record.packetInterval, sampler_record.spaceInterval);
+        } else {
+            dbg_printf("ID : %lld, algorithm : %u, packet interval: %u, packet space: %u\n", sampler_record.id, sampler_record.algorithm,
+                       sampler_record.packetInterval, sampler_record.spaceInterval);
+        }
 
         InsertSampler(fs, exporter, &sampler_record);
         return;
@@ -1511,7 +1595,7 @@ static void Process_ipfix_nbar_option_data(exporterDomain_t *exporter, FlowSourc
     // data size
     size_t data_size = nbarOption->id.length + nbarOption->name.length + nbarOption->desc.length;
     // size of record
-    size_t option_size = nbarOption->scopeSize + data_size;
+    size_t option_size = optionTemplate->optionSize;
     // number of records in data
     int numRecords = size_left / option_size;
     dbg_printf("[%u] nbar option data - records: %u, size: %zu\n", exporter->info.id, numRecords, option_size);
@@ -1540,7 +1624,7 @@ static void Process_ipfix_nbar_option_data(exporterDomain_t *exporter, FlowSourc
     // push nbar header
     AddArrayHeader(outBuff, nbarHeader, NbarRecordType, elementSize);
 
-    // put array info descripter next
+    // put array info descriptor next
     NbarAppInfo_t *NbarInfo = (NbarAppInfo_t *)(outBuff + sizeof(arrayRecordHeader_t));
     nbarHeader->size += sizeof(NbarAppInfo_t);
 
@@ -1548,13 +1632,13 @@ static void Process_ipfix_nbar_option_data(exporterDomain_t *exporter, FlowSourc
     NbarInfo->app_id_length = nbarOption->id.length;
     NbarInfo->app_name_length = nbarOption->name.length;
     NbarInfo->app_desc_length = nbarOption->desc.length;
+    dbg_printf("NBAR idLength: %u, nameLength: %u, descLength: %u\n", nbarOption->id.length, nbarOption->name.length, nbarOption->desc.length);
 
-    int cnt = 0;
+    dbg(int cnt = 0);
     while (size_left >= option_size) {
         // push nbar app info record
         uint8_t *p;
         PushArrayNextElement(nbarHeader, p, uint8_t);
-
         // copy data
         // id octet array
         memcpy(p, inBuff + nbarOption->id.offset, nbarOption->id.length);
@@ -1574,17 +1658,19 @@ static void Process_ipfix_nbar_option_data(exporterDomain_t *exporter, FlowSourc
         // description string
         memcpy(p, inBuff + nbarOption->desc.offset, nbarOption->desc.length);
         state = UTF8_ACCEPT;
-        if (validate_utf8(&state, (char *)p, nbarOption->name.length) == UTF8_REJECT) {
+        if (validate_utf8(&state, (char *)p, nbarOption->desc.length) == UTF8_REJECT) {
             LogError("Process_nbar_option: validate_utf8() %s line %d: %s", __FILE__, __LINE__, "invalid utf8 nbar description");
             err = 1;
         }
         p[nbarOption->desc.length - 1] = '\0';
-
-        cnt++;
 #ifdef DEVEL
+        cnt++;
         if (err == 0) {
-            printf("nbar record: %d: \n", cnt);
-            // PrintNbarRecord(nbarHeader);
+            printf("nbar record: %d, ", cnt);
+            uint8_t *u = (uint8_t *)(p - nbarOption->name.length - nbarOption->id.length);
+            for (int i = 0; i < nbarOption->id.length; i++) printf("%02X ", *((uint8_t *)u++));
+
+            printf("nbar record: %d, name: %s, desc: %s\n", cnt, p - nbarOption->name.length, p);
         } else {
             printf("Invalid nbar information - skip record\n");
         }
@@ -1605,7 +1691,7 @@ static void Process_ipfix_nbar_option_data(exporterDomain_t *exporter, FlowSourc
     fs->nffile->buff_ptr += nbarHeader->size;
 
     if (size_left > 7) {
-        LogInfo("Proces nbar data record - %u extra bytes", size_left);
+        LogInfo("Process nbar data record - %u extra bytes", size_left);
     }
     processed_records++;
 
@@ -1644,7 +1730,7 @@ static void Process_ifvrf_option_data(exporterDomain_t *exporter, FlowSource_t *
     // data size
     size_t data_size = nameOption->name.length + sizeof(uint32_t);
     // size of record
-    size_t option_size = nameOption->scopeSize + data_size;
+    size_t option_size = optionTemplate->optionSize;
     // number of records in data
     int numRecords = size_left / option_size;
     dbg_printf("[%u] name option data - records: %u, size: %zu\n", exporter->info.id, numRecords, option_size);
@@ -1673,14 +1759,14 @@ static void Process_ifvrf_option_data(exporterDomain_t *exporter, FlowSource_t *
     // push nbar header
     AddArrayHeader(outBuff, nameHeader, recordType, elementSize);
 
-    // put array info descripter next
+    // put array info descriptor next
     uint32_t *nameSize = (uint32_t *)(outBuff + sizeof(arrayRecordHeader_t));
     nameHeader->size += sizeof(uint32_t);
 
     // info record for each element in array
     *nameSize = nameOption->name.length;
 
-    int cnt = 0;
+    dbg(int cnt = 0);
     while (size_left >= option_size) {
         // push nbar app info record
         uint8_t *p;
@@ -1710,9 +1796,9 @@ static void Process_ifvrf_option_data(exporterDomain_t *exporter, FlowSource_t *
         } else {
             printf("Invalid name information - skip record\n");
         }
+        cnt++;
 #endif
         p += nameOption->name.length;
-        cnt++;
 
         // in case of an err we do no store this record
         if (err != 0) {
@@ -1729,7 +1815,7 @@ static void Process_ifvrf_option_data(exporterDomain_t *exporter, FlowSource_t *
     fs->nffile->buff_ptr += nameHeader->size;
 
     if (size_left > 7) {
-        LogInfo("Proces ifvrf data record - %u extra bytes", size_left);
+        LogInfo("Process ifvrf data record - %u extra bytes", size_left);
     }
     processed_records++;
 
