@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2022, Peter Haag
+ *  Copyright (c) 2009-2023, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -29,9 +29,12 @@
  *
  */
 
+#include "netflow_v9.h"
+
 #include <arpa/inet.h>
 #include <errno.h>
 #include <netinet/in.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -40,19 +43,13 @@
 #include <time.h>
 #include <unistd.h>
 
-#include "config.h"
-
-#ifdef HAVE_STDINT_H
-#include <stdint.h>
-#endif
-
 #include "bookkeeper.h"
 #include "collector.h"
+#include "config.h"
 #include "exporter.h"
 #include "fnf.h"
 #include "metric.h"
 #include "nbar.h"
-#include "netflow_v9.h"
 #include "nfdump.h"
 #include "nffile.h"
 #include "nfnet.h"
@@ -91,7 +88,7 @@ typedef struct exporterDomain_s {
 
     uint64_t packets;           // number of packets sent by this exporter
     uint64_t flows;             // number of flow records sent by this exporter
-    uint32_t sequence_failure;  // number of sequence failues
+    uint32_t sequence_failure;  // number of sequence failures
 
     // sampling information:
     // each flow source may have several sampler applied:
@@ -125,6 +122,8 @@ typedef struct exporterDomain_s {
 
 } exporterDomain_t;
 
+static int ExtensionsEnabled[MAXEXTENSIONS];
+
 static const struct v9TranslationMap_s {
     uint16_t id;  // v9 element id
 #define Stack_ONLY 0
@@ -248,8 +247,8 @@ static const struct v9TranslationMap_s {
     {NBAR_APPLICATION_ID, SIZEnbarAppID, ByteCopy, EXnbarAppID, OFFnbarAppID, STACK_NONE, "nbar application ID"},
 
     // sampling
-    {NF9_FLOW_SAMPLER_ID, Stack_ONLY, NumberCopy, EXnull, 0, STACK_SAMPLER, "sampler ID"},
-    {SELECTOR_ID, Stack_ONLY, NumberCopy, EXnull, 0, STACK_SAMPLER, "sampler ID"},
+    {NF9_FLOW_SAMPLER_ID, SIZEsampID, NumberCopy, EXsamplerInfoID, OFFsampID, STACK_SAMPLER, "sampler ID"},
+    {SELECTOR_ID, SIZEsampID, NumberCopy, EXsamplerInfoID, OFFsampID, STACK_SAMPLER, "sampler ID"},
 
     // End of table
     {0, 0, 0, 0, 0, STACK_NONE, NULL},
@@ -273,8 +272,6 @@ static int printRecord;
 static int32_t defaultSampling;
 
 /* local function prototypes */
-static void ProcessOptionFlowset(exporterDomain_t *exporter, FlowSource_t *fs, templateList_t *templateList, void *data_flowset);
-
 static void InsertSampler(FlowSource_t *fs, exporterDomain_t *exporter, sampler_record_t *sampler_record);
 
 static inline void Process_v9_templates(exporterDomain_t *exporter, void *template_flowset, FlowSource_t *fs);
@@ -297,19 +294,51 @@ static inline exporterDomain_t *getExporter(FlowSource_t *fs, uint32_t exporter_
 
 #include "nffile_inline.c"
 
-int Init_v9(int verbose, int32_t sampling) {
+int Init_v9(int verbose, int32_t sampling, char *extensionList) {
     printRecord = verbose > 2;
 
     defaultSampling = sampling;
 
-    int i;
-    for (i = 0; v9TranslationMap[i].name != NULL; i++) {
+    if (extensionList) {
+        // Disable all extensions
+        for (int i = 0; i < MAXEXTENSIONS; i++) {
+            ExtensionsEnabled[i] = 0;
+        }
+
+        // get enabled extensions from string
+        int extID = ScanExtension(extensionList);
+        while (extID > 0) {
+            dbg_printf("Enable extension %d\n", extID);
+            ExtensionsEnabled[extID] = 1;
+            extID = ScanExtension(NULL);
+        }
+
+        if (extID == -1) {
+            LogError("Failed to scan extension list.");
+            return 0;
+        }
+
+        // make sure extension 1 is enabled
+        ExtensionsEnabled[1] = 1;
+
+    } else {
+        // Enable all extensions
+        dbg_printf("Enable all extensions\n");
+        for (int i = 0; i < MAXEXTENSIONS; i++) {
+            ExtensionsEnabled[i] = 1;
+        }
+    }
+
+    int tagsEnabled = 0;
+    for (int i = 0; v9TranslationMap[i].name != NULL; i++) {
+        int extID = v9TranslationMap[i].extensionID;
+        if (ExtensionsEnabled[extID]) tagsEnabled++;
     }
 
     if (sampling < 0) {
-        LogInfo("Init v9: Max number of v9 tags: %u, overwrite sampling: %d", i, -defaultSampling);
+        LogInfo("Init v9: Max number of v9 tags enabled: %u, overwrite sampling: %d", tagsEnabled, -defaultSampling);
     } else {
-        LogInfo("Init v9: Max number of v9 tags: %u, default sampling: %d", i, defaultSampling);
+        LogInfo("Init v9: Max number of v9 tags enabled: %u, default sampling: %d", tagsEnabled, defaultSampling);
     }
 
     return 1;
@@ -336,7 +365,16 @@ static int LookupElement(uint16_t type, int EnterpriseNumber) {
 
     int i = 0;
     while (v9TranslationMap[i].name != NULL) {
-        if (v9TranslationMap[i].id == type) return i;
+        if (v9TranslationMap[i].id == type) {
+            int extID = v9TranslationMap[i].extensionID;
+            if (ExtensionsEnabled[extID]) {
+                return i;
+            } else {
+                dbg_printf("Extension %d not enabled\n", extID);
+                return -1;
+            }
+        }
+
         i++;
     }
 
@@ -410,7 +448,7 @@ static inline exporterDomain_t *getExporter(FlowSource_t *fs, uint32_t exporter_
         dbg_printf("Add static sampler for default sampling: %u\n", defaultSampling);
     }
 
-    LogInfo("Process_v9: New v9 exporter: SysID: %u, Domain: %u, IP: %s\n", (*e)->info.sysid, exporter_id, ipstr);
+    LogInfo("Process_v9: New v9 exporter: SysID: %u, Domain: %u, IP: %s", (*e)->info.sysid, exporter_id, ipstr);
 
     return (*e);
 
@@ -575,6 +613,7 @@ static void removeTemplate(exporterDomain_t *exporter, uint16_t id) {
     if (TestFlag(template->type, DATA_TEMPLATE)) {
         dataTemplate_t *dataTemplate = (dataTemplate_t *)template->data;
         ClearSequencer(&(dataTemplate->sequencer));
+        if (dataTemplate->extensionList) free(dataTemplate->extensionList);
     }
     free(template->data);
     free(template);
@@ -583,7 +622,7 @@ static void removeTemplate(exporterDomain_t *exporter, uint16_t id) {
 
 static inline void Process_v9_templates(exporterDomain_t *exporter, void *DataPtr, FlowSource_t *fs) {
     void *template;
-    uint32_t size_left, size_required, num_v9tags;
+    uint32_t size_left, size_required;
     int i;
 
     size_left = GET_FLOWSET_LENGTH(DataPtr);
@@ -614,8 +653,6 @@ static inline void Process_v9_templates(exporterDomain_t *exporter, void *DataPt
             return;
         }
 
-        num_v9tags = 0;  // number of optional v9 tags
-
         sequence_t *sequenceTable = malloc((count + 4) * sizeof(sequence_t));  // + 2 for IP and time received
         if (!sequenceTable) {
             LogError("Process_v9: malloc(): %s line %d: %s", __FILE__, __LINE__, strerror(errno));
@@ -633,10 +670,9 @@ static inline void Process_v9_templates(exporterDomain_t *exporter, void *DataPt
             p = p + 2;
             Length = Get_val16(p);
             p = p + 2;
-            num_v9tags++;
 
             int index = LookupElement(Type, EnterpriseNumber);
-            if (index < 0) {  // not found - enter skip seqence
+            if (index < 0) {  // not found - enter skip sequence
                 sequenceTable[numSequences].inputType = Type;
                 sequenceTable[numSequences].inputLength = Length;
                 sequenceTable[numSequences].extensionID = EXnull;
@@ -706,7 +742,7 @@ static inline void Process_v9_templates(exporterDomain_t *exporter, void *DataPt
         // update size left of this flowset
         size_left -= size_required;
         if (size_left < 4) {
-            // pading
+            // padding
             dbg_printf("Skip %u bytes padding\n", size_left);
             return;
         }
@@ -758,59 +794,29 @@ static inline void Process_v9_option_templates(exporterDomain_t *exporter, void
         return;
     }
 
-    int i;
-    uint16_t offset = 0;
     p = option_template + 6;  // start of length/type data
-    for (i = 0; i < nr_scopes; i++) {
-#ifdef DEVEL
-        uint16_t type = Get_val16(p);
-#endif
-        p = p + 2;
-
-        uint16_t length = Get_val16(p);
-        p = p + 2;
-        offset += length;
-#ifdef DEVEL
-        printf("Scope field: Type ");
-        switch (type) {
-            case 1:
-                printf("(1) - System");
-                break;
-            case 2:
-                printf("(2) - Interface");
-                break;
-            case 3:
-                printf("(3) - Line Card");
-                break;
-            case 4:
-                printf("(4) - NetFlow Cache");
-                break;
-            case 5:
-                printf("(5) - Template");
-                break;
-            default:
-                printf("(%u) - Unknown", type);
-                break;
-        }
-        printf(", length %u\n", length);
-#endif
-    }
-    uint16_t scopeSize = offset;
 
     struct samplerOption_s *samplerOption = &(optionTemplate->samplerOption);
     struct nbarOptionList_s *nbarOption = &(optionTemplate->nbarOption);
     struct nameOptionList_s *ifnameOptionList = &(optionTemplate->ifnameOption);
     struct nameOptionList_s *vrfnameOptionList = &(optionTemplate->vrfnameOption);
 
-    for (; i < (nr_scopes + nr_options); i++) {
+    uint16_t scopeSize = 0;
+    uint16_t offset = 0;
+    for (int i = 0; i < (nr_scopes + nr_options); i++) {
         uint16_t type = Get_val16(p);
         p = p + 2;
         uint16_t length = Get_val16(p);
         p = p + 2;
-        dbg_printf("Option field Type: %u, offset: %u, length %u\n", type, offset, length);
+        if (i < nr_scopes) {
+            scopeSize += length;
+            dbg_printf("Scope field Type: %u, offset: %u, length %u\n", type, offset, length);
+        } else {
+            dbg_printf("Option field Type: %u, offset: %u, length %u\n", type, offset, length);
+        }
 
         switch (type) {
-            // general sampling
+            // Old std sampling tags
             case NF9_SAMPLING_INTERVAL:  // #34
                 samplerOption->spaceInterval.length = length;
                 samplerOption->spaceInterval.offset = offset;
@@ -824,7 +830,8 @@ static inline void Process_v9_option_templates(exporterDomain_t *exporter, void
                 dbg_printf(" Sampling #35 found\n");
                 break;
 
-            // individual samplers
+            // New std sampling, individual sammplers (sampling ID)
+            // Map old individual samplers
             case NF9_FLOW_SAMPLER_ID:  // #48 deprecated - fall through
                 dbg_printf(" Sampling #48 found\n");
             case SELECTOR_ID:  // #302
@@ -917,8 +924,9 @@ static inline void Process_v9_option_templates(exporterDomain_t *exporter, void
         }
         offset += length;
     }
+    optionTemplate->optionSize = offset;
 
-    dbg_printf("\n[%u] Option flags: %llx\n", exporter->info.id, optionTemplate->flags);
+    dbg_printf("\n[%u] Option size: %llu, flags: %llx\n", exporter->info.id, optionTemplate->optionSize, optionTemplate->flags);
     if (optionTemplate->flags) {
         // if it exitsts - remove old template on exporter with same ID
         templateList_t *template = newTemplate(exporter, tableID);
@@ -928,11 +936,14 @@ static inline void Process_v9_option_templates(exporterDomain_t *exporter, void
         }
         template->data = optionTemplate;
 
-        if ((optionTemplate->flags & SAMPLERMASK) == SAMPLER306) {
-            dbg_printf("[%u] Sampler information found\n", exporter->info.id);
+        if ((optionTemplate->flags & SAMPLERFLAGS) == SAMPLERFLAGS) {
+            dbg_printf("[%u] New Sampler information found\n", exporter->info.id);
             SetFlag(template->type, SAMPLER_TEMPLATE);
-        } else if ((optionTemplate->flags & STDMASK) == STDSAMPLING34) {
-            dbg_printf("[%u] Std sampling information found\n", exporter->info.id);
+        } else if ((optionTemplate->flags & SAMPLERSTDFLAGS) == SAMPLERSTDFLAGS) {
+            dbg_printf("[%u] New std sampling information found\n", exporter->info.id);
+            SetFlag(template->type, SAMPLER_TEMPLATE);
+        } else if ((optionTemplate->flags & STDMASK) == STDFLAGS) {
+            dbg_printf("[%u] Old std sampling information found\n", exporter->info.id);
             SetFlag(template->type, SAMPLER_TEMPLATE);
         } else {
             dbg_printf("[%u] No Sampling information found\n", exporter->info.id);
@@ -998,9 +1009,9 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
     // reserve space in output stream for EXipReceivedVx
     uint32_t receivedSize = 0;
     if (fs->sa_family == PF_INET6)
-        receivedSize = EXipReceivedV6Size;
+        receivedSize = ExtensionsEnabled[EXipReceivedV6ID] ? EXipReceivedV6Size : 0;
     else
-        receivedSize = EXipReceivedV4Size;
+        receivedSize = ExtensionsEnabled[EXipReceivedV4ID] ? EXipReceivedV4Size : 0;
 
     while (size_left > 0) {
         void *outBuff;
@@ -1014,7 +1025,7 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
         uint32_t outRecordSize = CalcOutRecordSize(sequencer, inBuff, size_left);
         int buffAvail = CheckBufferSpace(fs->nffile, sizeof(recordHeaderV3_t) + outRecordSize + receivedSize);
         if (buffAvail == 0) {
-            // this should really never occur, because the buffer gets flushed ealier
+            // this should really never occur, because the buffer gets flushed earlier
             LogError("Process_v9: output buffer size error. Skip ipfix record processing");
             dbg_printf("Process_v9: output buffer size error. Skip ipfix record processing");
             return;
@@ -1057,7 +1068,7 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
                 LogVerbose("Process v9: Sequencer run - resize output buffer");
                 buffAvail = CheckBufferSpace(fs->nffile, buffAvail + 1);
                 if (buffAvail == 0) {
-                    // this should really never occur, because the buffer gets flushed ealier
+                    // this should really never occur, because the buffer gets flushed earlier
                     LogError("Process_v9: output buffer size error. Skip ipfix record processing");
                     dbg_printf("Process_v9: output buffer size error. Skip ipfix record processing");
                     return;
@@ -1073,14 +1084,22 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
 
         // add router IP
         if (fs->sa_family == PF_INET6) {
-            PushExtension(recordHeaderV3, EXipReceivedV6, ipReceivedV6);
-            ipReceivedV6->ip[0] = fs->ip.V6[0];
-            ipReceivedV6->ip[1] = fs->ip.V6[1];
-            dbg_printf("Add IPv6 route IP extension\n");
+            if (ExtensionsEnabled[EXipReceivedV6ID]) {
+                PushExtension(recordHeaderV3, EXipReceivedV6, ipReceivedV6);
+                ipReceivedV6->ip[0] = fs->ip.V6[0];
+                ipReceivedV6->ip[1] = fs->ip.V6[1];
+                dbg_printf("Add IPv6 route IP extension\n");
+            } else {
+                dbg_printf("IPv6 route IP extension not enabled\n");
+            }
         } else {
-            PushExtension(recordHeaderV3, EXipReceivedV4, ipReceivedV4);
-            ipReceivedV4->ip = fs->ip.V4;
-            dbg_printf("Add IPv4 route IP extension\n");
+            if (ExtensionsEnabled[EXipReceivedV4ID]) {
+                PushExtension(recordHeaderV3, EXipReceivedV4, ipReceivedV4);
+                ipReceivedV4->ip = fs->ip.V4;
+                dbg_printf("Add IPv4 route IP extension\n");
+            } else {
+                dbg_printf("IPv4 route IP extension not enabled\n");
+            }
         }
 
         dbg_printf("Record: %u elements, size: %u\n", recordHeaderV3->numElements, recordHeaderV3->size);
@@ -1112,6 +1131,11 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
             sampler = sampler->next;
         }
 
+        EXsamplerInfo_t *samplerInfo = (EXsamplerInfo_t *)sequencer->offsetCache[EXsamplerInfoID];
+        if (samplerInfo) {
+            samplerInfo->exporter_sysid = exporter->info.sysid;
+        }
+
         if (overwriteSampler) {
             // hard overwrite sampling
             packetInterval = overwriteSampler->record.packetInterval;
@@ -1192,11 +1216,9 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
                     if (stack[STACK_ICMP] != 0) {
                         // icmp type/code element #32
                         genericFlow->dstPort = stack[STACK_ICMP];
-                    } else if (stack[STACK_ICMP_TYPE] || stack[STACK_ICMP_CODE]) {
+                    } else if (stack[STACK_ICMP_TYPE] != 0 || stack[STACK_ICMP_CODE] != 0) {
                         // icmp type and code elements #176 #177 #178 #179
-                        genericFlow->dstPort = 256 * stack[STACK_ICMP_TYPE] + stack[STACK_ICMP_CODE];
-                    } else {
-                        genericFlow->dstPort = 0;
+                        genericFlow->dstPort = (stack[STACK_ICMP_TYPE] << 8) + stack[STACK_ICMP_CODE];
                     }
                     break;
                 case IPPROTO_TCP:
@@ -1239,11 +1261,19 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
         EXnselCommon_t *nselCommon = sequencer->offsetCache[EXnselCommonID];
         if (nselCommon) {
             nselCommon->msecEvent = stack[STACK_MSEC];
+            if (genericFlow) {
+                genericFlow->msecFirst = stack[STACK_MSEC];
+                genericFlow->msecLast = stack[STACK_MSEC];
+            }
             SetFlag(recordHeaderV3->flags, V3_FLAG_EVENT);
         }
         EXnelCommon_t *nelCommon = sequencer->offsetCache[EXnelCommonID];
         if (nelCommon) {
             nelCommon->msecEvent = stack[STACK_MSEC];
+            if (genericFlow) {
+                genericFlow->msecFirst = stack[STACK_MSEC];
+                genericFlow->msecLast = stack[STACK_MSEC];
+            }
             SetFlag(recordHeaderV3->flags, V3_FLAG_EVENT);
         }
 
@@ -1290,10 +1320,9 @@ static inline void Process_v9_sampler_option_data(exporterDomain_t *exporter, Fl
     optionTemplate_t *optionTemplate = (optionTemplate_t *)template->data;
     struct samplerOption_s *samplerOption = &(optionTemplate->samplerOption);
 
-    if ((optionTemplate->flags & SAMPLERMASK) != 0) {
+    if ((optionTemplate->flags & SAMPLERSTDFLAGS) != 0) {
         sampler_record_t sampler_record = {0};
 
-        sampler_record.id = 0;
         if (CHECK_OPTION_DATA(size_left, samplerOption->id)) {
             sampler_record.id = Get_val(in, samplerOption->id.offset, samplerOption->id.length);
         }
@@ -1319,9 +1348,16 @@ static inline void Process_v9_sampler_option_data(exporterDomain_t *exporter, Fl
                 LogError("Process_v9_option: Zero sampling interval -> sampling == 1", __FILE__, __LINE__);
             }
         }
+
         dbg_printf("Extracted Sampler data:\n");
-        dbg_printf("ID : %lld, algorithm : %u, packet interval: %u, packet space: %u\n", sampler_record.id, sampler_record.algorithm,
-                   sampler_record.packetInterval, sampler_record.spaceInterval);
+        if (sampler_record.id == 0) {
+            sampler_record.id = SAMPLER_GENERIC;
+            dbg_printf("New std sampler: algorithm : %u, packet interval: %u, packet space: %u\n", sampler_record.algorithm,
+                       sampler_record.packetInterval, sampler_record.spaceInterval);
+        } else {
+            dbg_printf("ID : %lld, algorithm : %u, packet interval: %u, packet space: %u\n", sampler_record.id, sampler_record.algorithm,
+                       sampler_record.packetInterval, sampler_record.spaceInterval);
+        }
 
         InsertSampler(fs, exporter, &sampler_record);
         return;
@@ -1365,7 +1401,7 @@ static void Process_v9_nbar_option_data(exporterDomain_t *exporter, FlowSource_t
     // data size
     size_t data_size = nbarOption->id.length + nbarOption->name.length + nbarOption->desc.length;
     // size of record
-    size_t option_size = nbarOption->scopeSize + data_size;
+    size_t option_size = optionTemplate->optionSize;
     // number of records in data
     int numRecords = size_left / option_size;
     dbg_printf("[%u] nbar option data - records: %u, size: %zu\n", exporter->info.id, numRecords, option_size);
@@ -1394,7 +1430,7 @@ static void Process_v9_nbar_option_data(exporterDomain_t *exporter, FlowSource_t
     // push nbar header
     AddArrayHeader(outBuff, nbarHeader, NbarRecordType, elementSize);
 
-    // put array info descripter next
+    // put array info descriptor next
     NbarAppInfo_t *NbarInfo = (NbarAppInfo_t *)(outBuff + sizeof(arrayRecordHeader_t));
     nbarHeader->size += sizeof(NbarAppInfo_t);
 
@@ -1403,7 +1439,7 @@ static void Process_v9_nbar_option_data(exporterDomain_t *exporter, FlowSource_t
     NbarInfo->app_name_length = nbarOption->name.length;
     NbarInfo->app_desc_length = nbarOption->desc.length;
 
-    int cnt = 0;
+    dbg(int cnt = 0);
     while (size_left >= option_size) {
         // push nbar app info record
         uint8_t *p;
@@ -1434,11 +1470,10 @@ static void Process_v9_nbar_option_data(exporterDomain_t *exporter, FlowSource_t
         }
         p[nbarOption->desc.length - 1] = '\0';
 
-        cnt++;
 #ifdef DEVEL
+        cnt++;
         if (err == 0) {
-            printf("nbar record: %d: \n", cnt);
-            // PrintNbarRecord(nbarHeader);
+            printf("nbar record: %d, name: %s, desc: %s\n", cnt, p - nbarOption->name.length, p);
         } else {
             printf("Invalid nbar information - skip record\n");
         }
@@ -1459,7 +1494,7 @@ static void Process_v9_nbar_option_data(exporterDomain_t *exporter, FlowSource_t
     fs->nffile->buff_ptr += nbarHeader->size;
 
     if (size_left > 7) {
-        LogVerbose("Proces nbar data record - %u extra bytes", size_left);
+        LogVerbose("Process nbar data record - %u extra bytes", size_left);
     }
     processed_records++;
 
@@ -1499,7 +1534,7 @@ static void Process_v9_ifvrf_option_data(exporterDomain_t *exporter, FlowSource_
     // data size
     size_t data_size = nameOption->name.length + sizeof(uint32_t);
     // size of record
-    size_t option_size = nameOption->scopeSize + data_size;
+    size_t option_size = optionTemplate->optionSize;
     // number of records in data
     int numRecords = size_left / option_size;
     dbg_printf("[%u] name option data - records: %u, size: %zu\n", exporter->info.id, numRecords, option_size);
@@ -1528,14 +1563,14 @@ static void Process_v9_ifvrf_option_data(exporterDomain_t *exporter, FlowSource_
     // push nbar header
     AddArrayHeader(outBuff, nameHeader, recordType, elementSize);
 
-    // put array info descripter next
+    // put array info descriptor next
     uint32_t *nameSize = (uint32_t *)(outBuff + sizeof(arrayRecordHeader_t));
     nameHeader->size += sizeof(uint32_t);
 
     // info record for each element in array
     *nameSize = nameOption->name.length;
 
-    int cnt = 0;
+    dbg(int cnt = 0);
     while (size_left >= option_size) {
         // push nbar app info record
         uint8_t *p;
@@ -1565,9 +1600,9 @@ static void Process_v9_ifvrf_option_data(exporterDomain_t *exporter, FlowSource_
         } else {
             printf("Invalid name information - skip record\n");
         }
+        cnt++;
 #endif
         p += nameOption->name.length;
-        cnt++;
 
         // in case of an err we do no store this record
         if (err != 0) {
@@ -1584,7 +1619,7 @@ static void Process_v9_ifvrf_option_data(exporterDomain_t *exporter, FlowSource_
     fs->nffile->buff_ptr += nameHeader->size;
 
     if (size_left > 7) {
-        LogVerbose("Proces ifvrf data record - %u extra bytes", size_left);
+        LogVerbose("Process ifvrf data record - %u extra bytes", size_left);
     }
     processed_records++;
 
