@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2022, Peter Haag
+ *  Copyright (c) 2009-2023, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -41,12 +41,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/mman.h>
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/types.h>
+#include <sys/uio.h>
 #include <sys/wait.h>
 #include <time.h>
 
@@ -74,7 +74,10 @@
 #include "nfstatfile.h"
 #include "nfxV3.h"
 #include "pidfile.h"
+#include "privsep.h"
+#include "repeater.h"
 #include "util.h"
+#include "version.h"
 
 #ifdef HAVE_FTS_H
 #include <fts.h>
@@ -91,7 +94,6 @@
 
 #define DEFAULTCISCOPORT "9995"
 
-static void *shmem = NULL;
 static int verbose = 0;
 
 // Define a generic type to get data from socket or pcap file
@@ -101,21 +103,20 @@ typedef ssize_t (*packet_function_t)(int, void *, size_t, int, struct sockaddr *
 static FlowSource_t *FlowSource;
 
 static int done = 0;
-static int launcher_alive, periodic_trigger, launcher_pid;
-
-static const char *nfdump_version = VERSION;
+static int gotSIGCHLD = 0;
+static int periodic_trigger;
 
 /* Local function Prototypes */
 static void usage(char *name);
 
-static void kill_launcher(int pid);
+static void signalPrivsepChild(pid_t child_pid, int pfd);
 
 static void IntHandler(int signal);
 
 static inline FlowSource_t *GetFlowSource(struct sockaddr_storage *ss);
 
-static void run(packet_function_t receive_packet, int socket, repeater_t *repeater, time_t twin, time_t t_begin, int report_seq, int use_subdirs,
-                char *time_extension, int compress);
+static void run(packet_function_t receive_packet, int socket, int pfd, int rfd, time_t twin, time_t t_begin, int use_subdirs, char *time_extension,
+                int compress);
 
 /* Functions */
 static void usage(char *name) {
@@ -128,6 +129,10 @@ static void usage(char *name) {
         "-b host\t\tbind socket to host/IP addr\n"
         "-J mcastgroup\tJoin multicast group <mcastgroup>\n"
         "-p portnum\tlisten on port portnum\n"
+#ifdef PCAP
+        "-f pcapfile\tRead network data from pcap file.\n"
+        "-d device\tRead network data from device (interface).\n"
+#endif
         "-w flowdir \tset the output directory to store the flows.\n"
         "-C <file>\tRead optional config file.\n"
         "-S subdir\tSub directory format. see nfcapd(1) for format\n"
@@ -138,6 +143,7 @@ static void usage(char *name) {
         "-M dir \t\tSet the output directory for dynamic sources.\n"
         "-P pidfile\tset the PID file\n"
         "-R IP[/port]\tRepeat incoming packets to IP address/port. Max 8 repeaters.\n"
+        "-A\t\tEnable source address spoofing for packet repeater -R.\n"
         "-s rate\tset default sampling rate (default 1)\n"
         "-x process\tlaunch process after a new file becomes available\n"
         "-z\t\tLZO compress flows in output file.\n"
@@ -150,47 +156,39 @@ static void usage(char *name) {
         "-v\t\tIncrease verbose level.\n"
         "-4\t\tListen on IPv4 (default).\n"
         "-6\t\tListen on IPv6.\n"
+        "-X <extlist>\t',' separated list of extensions (numbers). Default all extensions.\n"
         "-V\t\tPrint version and exit.\n"
         "-Z\t\tAdd timezone offset to filename.\n",
         name);
 }  // End of usage
 
-void kill_launcher(int pid) {
-    int stat, i;
-    pid_t ret;
+static void signalPrivsepChild(pid_t child_pid, int pfd) {
+    if (pfd == 0) return;
 
-    if (pid == 0) return;
+    message_t message;
+    message.type = PRIVMSG_EXIT;
+    message.length = sizeof(message);
+    ssize_t ret = write(pfd, &message, sizeof(message));
 
-    if (launcher_alive) {
-        LogInfo("Signal launcher[%i] to terminate.", pid);
-        kill(pid, SIGTERM);
-
-        // wait for launcher to teminate
-        for (i = 0; i < LAUNCHER_TIMEOUT; i++) {
-            if (!launcher_alive) break;
-            sleep(1);
-        }
-        if (i >= LAUNCHER_TIMEOUT) {
-            LogError("Launcher does not want to terminate - signal again");
-            kill(pid, SIGTERM);
-            sleep(1);
-        }
-    } else {
-        LogError("launcher[%i] already dead", pid);
+    if (ret < 0) {
+        LogError("Failed to send exit message for privsep child. pipe write: %s", strerror(errno));
+        kill(child_pid, SIGTERM);
     }
 
-    if ((ret = waitpid(pid, &stat, 0)) == -1) {
-        LogError("wait for launcher failed: %s %i", strerror(errno), ret);
+    int stat = 0;
+    if ((ret = waitpid(child_pid, &stat, 0)) == -1) {
+        if (!gotSIGCHLD) LogError("wait for privsep child failed: %s", strerror(errno));
     } else {
         if (WIFEXITED(stat)) {
-            LogInfo("launcher exit status: %i", WEXITSTATUS(stat));
+            LogInfo("privsep child exit status: %i", WEXITSTATUS(stat));
         }
         if (WIFSIGNALED(stat)) {
-            LogError("launcher terminated due to signal %i", WTERMSIG(stat));
+            LogError("privsep child terminated due to signal %i", WTERMSIG(stat));
         }
+        LogVerbose("privsep child terminated with status: 0x%x", stat);
     }
 
-}  // End of kill_launcher
+}  // End of signalPrivsepChild
 
 static void IntHandler(int signal) {
     switch (signal) {
@@ -203,7 +201,9 @@ static void IntHandler(int signal) {
             done = 1;
             break;
         case SIGCHLD:
-            launcher_alive = 0;
+            gotSIGCHLD++;
+            break;
+        case SIGPIPE:
             break;
         default:
             // ignore everything we don't know
@@ -212,6 +212,26 @@ static void IntHandler(int signal) {
 
 } /* End of IntHandler */
 
+static void ChildDied(void) {
+    if (gotSIGCHLD) {
+        int stat = 0;
+        pid_t pid = waitpid(-1, &stat, 0);
+        if (pid == -1) {
+            if (!gotSIGCHLD) LogError("wait for privsep child failed: %s", strerror(errno));
+        } else {
+            if (WIFEXITED(stat)) {
+                LogInfo("privsep child[%u] exit status: %i", pid, WEXITSTATUS(stat));
+            }
+            if (WIFSIGNALED(stat)) {
+                LogError("privsep child[%u] terminated due to signal %i", pid, WTERMSIG(stat));
+            }
+            LogError("privsep child[%u] terminated with status: pid, 0x%x", pid, stat);
+        }
+        gotSIGCHLD--;
+    }
+
+}  // End of ChildDied
+
 static void format_file_block_header(dataBlock_t *header) {
     printf("File Block Header: type: %u, size: %u, NumRecords: %u\n", header->type, header->size, header->NumRecords);
 }  // End of format_file_block_header
@@ -219,19 +239,52 @@ static void format_file_block_header(dataBlock_t *header) {
 #include "collector_inline.c"
 #include "nffile_inline.c"
 
-static void run(packet_function_t receive_packet, int socket, repeater_t *repeater, time_t twin, time_t t_begin, int report_seq, int use_subdirs,
-                char *time_extension, int compress) {
+static int SendRepeaterMessage(int fd, void *in_buff, size_t cnt, struct sockaddr_storage *sender, socklen_t sender_size) {
+    message_t message;
+    message.type = PRIVMSG_REPEAT;
+    message.length = cnt + sizeof(message_t);
+
+    repeater_message_t repeater_message;
+    repeater_message.packet_size = cnt;
+    repeater_message.storage_size = sender_size;
+    repeater_message.addr = *sender;
+
+    struct iovec vector[3];
+    size_t len;
+    vector[0].iov_base = &message;
+    vector[0].iov_len = sizeof(message_t);
+    len = sizeof(message_t);
+
+    vector[1].iov_base = &repeater_message;
+    vector[1].iov_len = sizeof(repeater_message_t);
+    len += sizeof(repeater_message_t);
+
+    vector[2].iov_base = in_buff;
+    vector[2].iov_len = cnt;
+    len += cnt;
+
+    message.length = len;
+    ssize_t ret = writev(fd, vector, 3);
+    if (ret < 0) {
+        LogError("Failed to send repeater message: %s", strerror(errno));
+        return errno;
+    } else {
+        dbg_printf("Sent message to repeater: %u\n", message.length);
+    }
+    return 0;
+}  // End of SendRepeaterMessage
+
+static void run(packet_function_t receive_packet, int socket, int pfd, int rfd, time_t twin, time_t t_begin, int use_subdirs, char *time_extension,
+                int compress) {
     common_flow_header_t *nf_header;
     FlowSource_t *fs;
     struct sockaddr_storage nf_sender;
     socklen_t nf_sender_size = sizeof(nf_sender);
     time_t t_start, t_now;
-    uint64_t export_packets;
-    uint32_t blast_cnt, blast_failures, ignored_packets;
+    uint32_t ignored_packets;
     uint16_t version;
     ssize_t cnt;
     void *in_buff;
-    srecord_t *commbuff;
 
     in_buff = malloc(NETWORK_INPUT_BUFF_SIZE);
     if (!in_buff) {
@@ -239,15 +292,13 @@ static void run(packet_function_t receive_packet, int socket, repeater_t *repeat
         return;
     }
 
-    // init vars
-    commbuff = (srecord_t *)shmem;
     nf_header = (common_flow_header_t *)in_buff;
 
     // Init each netflow source output data buffer
     fs = FlowSource;
     while (fs) {
         // prepare file
-        fs->nffile = OpenNewFile(fs->current, NULL, compress, NOT_ENCRYPTED);
+        fs->nffile = OpenNewFile(fs->current, NULL, CREATOR_NFCAPD, compress, NOT_ENCRYPTED);
         if (!fs->nffile) {
             return;
         }
@@ -262,7 +313,6 @@ static void run(packet_function_t receive_packet, int socket, repeater_t *repeat
         fs = fs->next;
     }
 
-    export_packets = blast_cnt = blast_failures = 0;
     t_start = t_begin;
 
     cnt = 0;
@@ -279,9 +329,8 @@ static void run(packet_function_t receive_packet, int socket, repeater_t *repeat
      */
     while (1) {
         struct timeval tv;
-        int i;
 
-        /* read next bunch of data into beginn of input buffer */
+        /* read next bunch of data into begin of input buffer */
         if (!done) {
 #ifdef PCAP
             // Debug code to read from pcap file, or from socket
@@ -297,16 +346,6 @@ static void run(packet_function_t receive_packet, int socket, repeater_t *repeat
                 LogError("ERROR: recvfrom: %s", strerror(errno));
                 continue;
             }
-
-            i = 0;
-            while (repeater[i].hostname && (i < MAX_REPEATERS)) {
-                ssize_t len;
-                len = sendto(repeater[i].sockfd, in_buff, cnt, 0, (struct sockaddr *)&(repeater[i].addr), repeater[i].addrlen);
-                if (len < 0) {
-                    LogError("ERROR: sendto(): %s", strerror(errno));
-                }
-                i++;
-            }
         }
 
         /* Periodic file renaming, if time limit reached or if we are done.  */
@@ -316,7 +355,7 @@ static void run(packet_function_t receive_packet, int socket, repeater_t *repeat
 
         if (((t_now - t_start) >= twin) || done) {
             struct tm *now;
-            char *subdir, fmt[MAXTIMESTRING];
+            char *subdir, fmt[32];
 
             alarm(0);
             now = localtime(&t_start);
@@ -380,13 +419,11 @@ static void run(packet_function_t receive_packet, int socket, repeater_t *repeat
                 // file otherwise, we will loose flows and can not continue collecting new flows
                 if (RenameAppend(fs->current, nfcapd_filename) < 0) {
                     LogError("Ident: %s, Can't rename dump file: %s", fs->Ident, strerror(errno));
-                    if (launcher_pid) commbuff->failed = 1;
 
                     // we do not update the books here, as the file failed to rename properly
                     // otherwise the books may be wrong
                 } else {
                     struct stat fstat;
-                    if (launcher_pid) commbuff->failed = 0;
 
                     // Update books
                     stat(nfcapd_filename, &fstat);
@@ -406,7 +443,7 @@ static void run(packet_function_t receive_packet, int socket, repeater_t *repeat
                 fs->msecLast = 0;
 
                 if (!done) {
-                    fs->nffile = OpenNewFile(fs->current, fs->nffile, compress, NOT_ENCRYPTED);
+                    fs->nffile = OpenNewFile(fs->current, fs->nffile, CREATOR_NFCAPD, compress, NOT_ENCRYPTED);
                     if (!fs->nffile) {
                         LogError("killed due to fatal error: ident: %s", fs->Ident);
                         break;
@@ -417,33 +454,21 @@ static void run(packet_function_t receive_packet, int socket, repeater_t *repeat
                     FlushStdRecords(fs);
                 }
 
+                // trigger launcher if required
+                if (pfd) {
+                    // Send launcher message
+                    if (SendLauncherMessage(pfd, t_start, subdir, fmt, fs->datadir, fs->Ident) < 0) {
+                        LogError("Failed to send launcher message");
+                    } else {
+                        LogVerbose("Send launcher message");
+                    }
+                }
+
                 // next flow source
                 fs = fs->next;
 
             }  // end of while (fs)
 
-            // trigger launcher if required
-            if (launcher_pid) {
-                // Signal launcher
-
-                strncpy(commbuff->tstring, fmt, MAXTIMESTRING);
-                commbuff->tstring[MAXTIMESTRING - 1] = '\0';
-
-                commbuff->tstamp = t_start;
-                if (subdir) {
-                    snprintf(commbuff->fname, MAXPATHLEN - 1, "%s/nfcapd.%s", subdir, fmt);
-                } else {
-                    snprintf(commbuff->fname, MAXPATHLEN - 1, "nfcapd.%s", fmt);
-                }
-                commbuff->fname[MAXPATHLEN - 1] = '\0';
-
-                if (launcher_alive) {
-                    LogVerbose("Signal launcher");
-                    kill(launcher_pid, SIGHUP);
-                } else
-                    LogError("ERROR: Launcher died unexpectedly!");
-            }
-
             if (ignored_packets) LogInfo("Total ignored packets: %u", ignored_packets);
             ignored_packets = 0;
 
@@ -466,12 +491,13 @@ static void run(packet_function_t receive_packet, int socket, repeater_t *repeat
                 periodic_trigger = 0;
                 continue;
             }
-            if (done)
+            if (done) {
                 // signaled to terminate - exit from loop
                 break;
-            else {
-                /* this should never be executed as it should be caught in other places */
-                LogError("error condition in '%s', line '%d', cnt: %i", __FILE__, __LINE__, (int)cnt);
+            } else {
+                // A child could have died
+                ChildDied();
+                LogError("recvfrom() error in '%s', line '%d', cnt: %d:, %s", __FILE__, __LINE__, cnt, strerror(errno));
                 continue;
             }
         }
@@ -479,6 +505,15 @@ static void run(packet_function_t receive_packet, int socket, repeater_t *repeat
         /* enough data? */
         if (cnt == 0) continue;
 
+        // repeat this packet
+        if (rfd) {
+            if (SendRepeaterMessage(rfd, in_buff, cnt, &nf_sender, nf_sender_size) != 0) {
+                LogError("Disable packet repeater due to errors");
+                close(rfd);
+                rfd = 0;
+            }
+        }
+
         // get flow source record for current packet, identified by sender IP address
         fs = GetFlowSource(&nf_sender);
         if (fs == NULL) {
@@ -488,12 +523,12 @@ static void run(packet_function_t receive_packet, int socket, repeater_t *repeat
                 ignored_packets++;
                 continue;
             }
-            if (InitBookkeeper(&fs->bookkeeper, fs->datadir, getpid(), launcher_pid) != BOOKKEEPER_OK) {
+            if (InitBookkeeper(&fs->bookkeeper, fs->datadir, getpid()) != BOOKKEEPER_OK) {
                 LogError("Failed to initialise bookkeeper for new source");
                 // fatal error
                 return;
             }
-            fs->nffile = OpenNewFile(fs->current, NULL, compress, NOT_ENCRYPTED);
+            fs->nffile = OpenNewFile(fs->current, NULL, CREATOR_NFCAPD, compress, NOT_ENCRYPTED);
             if (!fs->nffile) {
                 LogError("Failed to open new collector file");
                 return;
@@ -539,12 +574,8 @@ static void run(packet_function_t receive_packet, int socket, repeater_t *repeat
         }
         // each Process_xx function has to process the entire input buffer, therefore it's empty
         // now.
-        export_packets++;
     }
 
-    if (verbose && blast_failures) {
-        LogError("Total missed packets: %u", blast_failures);
-    }
     free(in_buff);
 
     fs = FlowSource;
@@ -558,28 +589,25 @@ static void run(packet_function_t receive_packet, int socket, repeater_t *repeat
 
 int main(int argc, char **argv) {
     char *bindhost, *datadir, *launch_process;
-    char *userid, *groupid, *checkptr, *listenport, *mcastgroup;
+    char *userid, *groupid, *listenport, *mcastgroup;
     char *Ident, *dynFlowDir, *time_extension, *pidfile, *configFile, *metricSocket;
+    char *extensionList;
     packet_function_t receive_packet;
     repeater_t repeater[MAX_REPEATERS];
     FlowSource_t *fs;
-    struct sigaction act;
     int family, bufflen, metricInterval;
-    time_t twin, t_start;
-    int sock, do_daemonize, expire, spec_time_extension, report_sequence;
-    int subdir_index, sampling_rate, compress;
-    int c, i;
+    time_t twin;
+    int sock, do_daemonize, expire, spec_time_extension;
+    int subdir_index, sampling_rate, compress, srcSpoofing;
 #ifdef PCAP
     char *pcap_file = NULL;
+    char *pcap_device = NULL;
 #endif
 
     receive_packet = recvfrom;
     verbose = do_daemonize = 0;
     bufflen = 0;
     family = AF_UNSPEC;
-    launcher_pid = 0;
-    launcher_alive = 0;
-    report_sequence = 0;
     listenport = DEFAULTCISCOPORT;
     bindhost = NULL;
     mcastgroup = NULL;
@@ -595,17 +623,17 @@ int main(int argc, char **argv) {
     sampling_rate = 1;
     compress = NOT_COMPRESSED;
     memset((void *)&repeater, 0, sizeof(repeater));
-    for (i = 0; i < MAX_REPEATERS; i++) {
-        repeater[i].family = AF_UNSPEC;
-    }
+    srcSpoofing = 0;
     configFile = NULL;
     Ident = "none";
     FlowSource = NULL;
     dynFlowDir = NULL;
     metricSocket = NULL;
     metricInterval = 60;
+    extensionList = NULL;
 
-    while ((c = getopt(argc, argv, "46B:b:C:DeEf:g:hI:i:jJ:l:m:M:n:p:P:rRs:S:t:T:u:vVw:x:yzZ")) != EOF) {
+    int c;
+    while ((c = getopt(argc, argv, "46AB:b:C:d:DeEf:g:hI:i:jJ:l:m:M:n:p:P:R:s:S:t:T:u:vVw:x:X:yzZ")) != EOF) {
         switch (c) {
             case 'h':
                 usage(argv[0]);
@@ -629,8 +657,8 @@ int main(int argc, char **argv) {
             case 'e':
                 expire = 1;
                 break;
-            case 'f': {
 #ifdef PCAP
+            case 'f': {
                 struct stat fstat;
                 pcap_file = optarg;
                 stat(pcap_file, &fstat);
@@ -638,10 +666,17 @@ int main(int argc, char **argv) {
                     LogError("Not a regular file: %s", pcap_file);
                     exit(254);
                 }
-#else
-                LogError("PCAP reader not compiled! Option ignored!");
-#endif
             } break;
+            case 'd':
+                CheckArgLen(optarg, 32);
+                pcap_device = strdup(optarg);
+                break;
+#else
+            case 'f':
+            case 'd':
+                LogError("Reading data from pcap file/device not compiled! Option ignored!");
+                break;
+#endif
             case 'E':
                 verbose = 3;
                 break;
@@ -649,7 +684,7 @@ int main(int argc, char **argv) {
                 if (verbose < 4) verbose++;
                 break;
             case 'V':
-                printf("%s: Version: %s\n", argv[0], nfdump_version);
+                printf("%s: %s\n", argv[0], versionString());
                 exit(EXIT_SUCCESS);
                 break;
             case 'D':
@@ -686,16 +721,19 @@ int main(int argc, char **argv) {
                 }
                 break;
             case 'n':
-                if (!AddFlowSourceString(&FlowSource, optarg)) {
+                CheckArgLen(optarg, MAXPATHLEN);
+                if (!AddFlowSourceString(&FlowSource, strdup(optarg))) {
                     LogError("Failed to add flow source");
                     exit(EXIT_FAILURE);
                 }
                 break;
-            case 'B':
+            case 'B': {
+                char *checkptr = NULL;
                 bufflen = strtol(optarg, &checkptr, 10);
                 if ((checkptr != NULL && *checkptr == 0) && bufflen > 0) break;
                 LogError("Argument error for -B");
                 exit(EXIT_FAILURE);
+            } break;
             case 'b':
                 bindhost = optarg;
                 break;
@@ -712,16 +750,15 @@ int main(int argc, char **argv) {
                 }
                 break;
             case 'R': {
-                char *port, *hostname;
-                char *p = strchr(optarg, '/');
-                int i = 0;
+                CheckArgLen(optarg, 128);
+                char *hostname = strdup(optarg);
+                char *port = DEFAULTCISCOPORT;
+                char *p = strchr(hostname, '/');
                 if (p) {
                     *p++ = '\0';
-                    port = strdup(p);
-                } else {
-                    port = DEFAULTCISCOPORT;
+                    port = p;
                 }
-                hostname = strdup(optarg);
+                int i = 0;
                 while (repeater[i].hostname && (i < MAX_REPEATERS)) i++;
                 if (i == MAX_REPEATERS) {
                     LogError("Too many packet repeaters! Max: %i repeaters allowed", MAX_REPEATERS);
@@ -732,8 +769,12 @@ int main(int argc, char **argv) {
 
                 break;
             }
-            case 'r':
-                report_sequence = 1;
+            case 'A':
+                srcSpoofing = 1;
+                if (RunAsRoot() == 0) {
+                    LogError("Src IP spoofing requires process to start as root");
+                    exit(EXIT_FAILURE);
+                }
                 break;
             case 's':
                 // a negative sampling rate is set as the overwrite sampling rate
@@ -773,8 +814,13 @@ int main(int argc, char **argv) {
                 }
                 break;
             case 'x':
+                CheckArgLen(optarg, 256);
                 launch_process = optarg;
                 break;
+            case 'X':
+                CheckArgLen(optarg, 128);
+                extensionList = strdup(optarg);
+                break;
             case 'j':
                 if (compress) {
                     LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression");
@@ -822,11 +868,30 @@ int main(int argc, char **argv) {
         }
     }
 
-    if (argc - optind > 1) {
-        usage(argv[0]);
+    if (!InitLog(do_daemonize, argv[0], SYSLOG_FACILITY, verbose)) {
         exit(EXIT_FAILURE);
     }
 
+    if ((argc - optind) >= 2) {
+        if (strcmp(argv[optind], "privsep") == 0) {
+            if (strcmp(argv[optind + 1], "launcher") == 0) {
+                dbg_printf("nfcapd privsep launched\n");
+                int ret = StartupLauncher(launch_process, expire);
+                exit(ret);
+            } else if (strcmp(argv[optind + 1], "repeater") == 0) {
+                dbg_printf("nfcapd repeater launched\n");
+                int ret = StartupRepeater(repeater, bufflen, srcSpoofing, userid, groupid);
+                exit(ret);
+            } else {
+                usage(argv[0]);
+                exit(EXIT_FAILURE);
+            }
+        } else {
+            usage(argv[0]);
+            exit(EXIT_FAILURE);
+        }
+    }
+
     if (ConfOpen(configFile, "nfcapd") < 0) exit(EXIT_FAILURE);
 
     if (datadir && !AddFlowSource(&FlowSource, Ident, ANYIP, datadir)) {
@@ -849,10 +914,6 @@ int main(int argc, char **argv) {
         exit(EXIT_FAILURE);
     }
 
-    if (!InitLog(do_daemonize, argv[0], SYSLOG_FACILITY, verbose)) {
-        exit(EXIT_FAILURE);
-    }
-
     if (!Init_nffile(NULL)) exit(254);
 
     if (expire && spec_time_extension) {
@@ -860,12 +921,22 @@ int main(int argc, char **argv) {
         exit(EXIT_FAILURE);
     }
 
-    // Debug code to read from pcap file
+// Debug code to read from pcap file
 #ifdef PCAP
     sock = 0;
     if (pcap_file) {
-        printf("Setup pcap reader\n");
-        setup_packethandler(pcap_file, NULL);
+        printf("Setup pcap file reader\n");
+        if (!setup_pcap_offline(pcap_file, NULL)) {
+            LogError("Setup pcap offline failed.");
+            exit(EXIT_FAILURE);
+        }
+        receive_packet = NextPacket;
+    } else if (pcap_device) {
+        printf("Setup pcap device reader\n");
+        if (!setup_pcap_live(pcap_device, NULL, bufflen)) {
+            LogError("Setup pcap device failed.");
+            exit(EXIT_FAILURE);
+        }
         receive_packet = NextPacket;
     } else
 #endif
@@ -879,19 +950,16 @@ int main(int argc, char **argv) {
         exit(EXIT_FAILURE);
     }
 
-    i = 0;
-    while (repeater[i].hostname && (i < MAX_REPEATERS)) {
-        repeater[i].sockfd =
-            Unicast_send_socket(repeater[i].hostname, repeater[i].port, repeater[i].family, bufflen, &repeater[i].addr, &repeater[i].addrlen);
-        if (repeater[i].sockfd <= 0) exit(EXIT_FAILURE);
-        LogVerbose("Replay flows to host: %s port: %s", repeater[i].hostname, repeater[i].port);
-        i++;
+    pid_t repeater_pid = 0;
+    int rfd = 0;
+    if (repeater[0].hostname) {
+        rfd = PrivsepFork(argc, argv, &repeater_pid, "repeater");
     }
 
     SetPriv(userid, groupid);
 
-    if (!Init_v1(verbose) || !Init_v5_v7(verbose, sampling_rate) || !Init_pcapd(verbose) || !Init_v9(verbose, sampling_rate) ||
-        !Init_IPFIX(verbose, sampling_rate)) {
+    if (!Init_v1(verbose) || !Init_v5_v7(verbose, sampling_rate) || !Init_pcapd(verbose) || !Init_v9(verbose, sampling_rate, extensionList) ||
+        !Init_IPFIX(verbose, sampling_rate, extensionList)) {
         exit(EXIT_FAILURE);
     }
 
@@ -900,7 +968,7 @@ int main(int argc, char **argv) {
         exit(EXIT_FAILURE);
     }
 
-    t_start = time(NULL);
+    time_t t_start = time(NULL);
     t_start = t_start - (t_start % twin);
 
     if (do_daemonize) {
@@ -917,38 +985,15 @@ int main(int argc, char **argv) {
         exit(EXIT_FAILURE);
     }
 
+    int launcher_pid = 0;
+    int pfd = 0;
     if (launch_process || expire) {
-        // create laucher comm memory struct
-        shmem = mmap(0, sizeof(srecord_t), PROT_READ | PROT_WRITE, MAP_ANON | MAP_SHARED, -1, 0);
-        if (shmem == MAP_FAILED) {
-            LogError("mmap() error in %s:%i: %s", __FILE__, __LINE__, strerror(errno));
-            close(sock);
-            exit(EXIT_FAILURE);
-        }
-
-        launcher_pid = fork();
-        switch (launcher_pid) {
-            case 0:
-                // child
-                close(sock);
-                launcher(shmem, FlowSource, launch_process, expire);
-                _exit(EXIT_SUCCESS);
-                break;
-            case -1:
-                LogError("fork() error: %s", strerror(errno));
-                if (pidfile) remove_pid(pidfile);
-                exit(EXIT_FAILURE);
-                break;
-            default:
-                // parent
-                launcher_alive = 1;
-                LogVerbose("Launcher[%i] forked", launcher_pid);
-        }
+        pfd = PrivsepFork(argc, argv, &launcher_pid, "launcher");
     }
 
     fs = FlowSource;
     while (fs) {
-        if (InitBookkeeper(&fs->bookkeeper, fs->datadir, getpid(), launcher_pid) != BOOKKEEPER_OK) {
+        if (InitBookkeeper(&fs->bookkeeper, fs->datadir, getpid()) != BOOKKEEPER_OK) {
             LogError("initialize bookkeeper failed");
 
             // release all already allocated bookkeepers
@@ -958,7 +1003,8 @@ int main(int argc, char **argv) {
                 fs = fs->next;
             }
             close(sock);
-            if (launcher_pid) kill_launcher(launcher_pid);
+            signalPrivsepChild(launcher_pid, pfd);
+            signalPrivsepChild(repeater_pid, rfd);
             if (pidfile) remove_pid(pidfile);
             exit(EXIT_FAILURE);
         }
@@ -967,6 +1013,7 @@ int main(int argc, char **argv) {
     }
 
     /* Signal handling */
+    struct sigaction act;
     memset((void *)&act, 0, sizeof(struct sigaction));
     act.sa_handler = IntHandler;
     sigemptyset(&act.sa_mask);
@@ -976,13 +1023,15 @@ int main(int argc, char **argv) {
     sigaction(SIGHUP, &act, NULL);
     sigaction(SIGALRM, &act, NULL);
     sigaction(SIGCHLD, &act, NULL);
+    sigaction(SIGPIPE, &act, NULL);
 
     LogInfo("Startup nfcapd.");
-    run(receive_packet, sock, repeater, twin, t_start, report_sequence, subdir_index, time_extension, compress);
+    run(receive_packet, sock, pfd, rfd, twin, t_start, subdir_index, time_extension, compress);
 
     // shutdown
     close(sock);
-    kill_launcher(launcher_pid);
+    signalPrivsepChild(launcher_pid, pfd);
+    signalPrivsepChild(repeater_pid, rfd);
     CloseMetric();
 
     fs = FlowSource;
