@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2022, Peter Haag
+ *  Copyright (c) 2009-2023, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -516,7 +516,7 @@ int Init_StatTable(void) {
 
 }  // End of Init_StatTable
 
-void Dispose_StatTable() { nfalloc_free(); }  // End of Dispose_Tables
+void Dispose_StatTable(void) { nfalloc_free(); }  // End of Dispose_Tables
 
 int SetStat(char *str, int *element_stat, int *flow_stat) {
     if (NumStats == MaxStats) {
@@ -1068,7 +1068,7 @@ void PrintElementStat(stat_record_t *sum_stat, outputParams_t *outputParams, Rec
             if (order & order_bit) {
                 SortElement_t *topN_element_list = StatTopN(outputParams->topN, &numflows, hash_num, order_index, direction);
 
-                // this output formating is pretty ugly - and needs to be cleaned up - improved
+                // this output formatting is pretty ugly - and needs to be cleaned up - improved
                 if (outputParams->mode == MODE_PLAIN && !outputParams->quiet) {
                     if (outputParams->topN != 0) {
                         printf("Top %i %s ordered by %s:\n", outputParams->topN, StatParameters[stat].HeaderInfo, order_mode[order_index].string);
@@ -1196,3 +1196,20 @@ static SortElement_t *StatTopN(int topN, uint32_t *count, int hash_num, int orde
     return topN_list;
 
 }  // End of StatTopN
+
+void ListPrintOrder(void) {
+    printf("Available print order:\n");
+    for (int i = 0; order_mode[i].string != NULL; i++) {
+        printf("%s ", order_mode[i].string);
+    }
+    printf("- See also nfdump(1)\n");
+}  // End of ListPrintOrder
+
+void ListStatTypes(void) {
+    printf("Available element statistics:");
+    for (int i = 0; StatParameters[i].statname != NULL; i++) {
+        if ((i & 0xf) == 0) printf("\n");
+        printf("%s ", StatParameters[i].statname);
+    }
+    printf("- See also nfdump(1)\n");
+}  // End of ListStatTypes
\ No newline at end of file
