@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2013-2022, Peter Haag
+ *  Copyright (c) 2013-2023, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -28,11 +28,18 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <arpa/inet.h>
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <libgen.h>
+#ifdef FIX_INCLUDE
+#include <sys/types.h>
+#endif
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
@@ -51,7 +58,6 @@
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
-#include <sys/time.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <time.h>
@@ -76,7 +82,9 @@
 #include "pcapdump.h"
 #include "pcaproc.h"
 #include "pidfile.h"
+#include "repeater.h"
 #include "util.h"
+#include "version.h"
 
 #define TIME_WINDOW 300
 #define PROMISC 1
@@ -91,8 +99,6 @@ static int done = 0;
  */
 #define PCAP_DUMPFILE "pcap.current"
 
-static const char *nfdump_version = VERSION;
-
 static int launcher_pid;
 static pthread_mutex_t m_done = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t terminate = PTHREAD_COND_INITIALIZER;
@@ -184,38 +190,18 @@ static int setup_pcap_file(packetParam_t *param, char *pcap_file, char *filter,
         }
     }
 
-    linkoffset = 0;
     linktype = pcap_datalink(handle);
     switch (linktype) {
         case DLT_RAW:
-            linkoffset = 0;
-            break;
         case DLT_PPP:
-            linkoffset = 2;
-            break;
         case DLT_PPP_SERIAL:
-            linkoffset = 4;
-            break;
         case DLT_NULL:
-            linkoffset = 4;
-            break;
         case DLT_LOOP:
-            linkoffset = 14;
-            break;
         case DLT_EN10MB:
-            linkoffset = 14;
-            break;
         case DLT_LINUX_SLL:
-            linkoffset = 16;
-            break;
         case DLT_IEEE802_11:
-            linkoffset = 22;
-            break;
         case DLT_NFLOG:
-            linkoffset = 0;
-            break;
         case DLT_PFLOG:
-            linkoffset = 0;
             break;
         default:
             LogError("Unsupported data link type %i", linktype);
@@ -223,7 +209,6 @@ static int setup_pcap_file(packetParam_t *param, char *pcap_file, char *filter,
     }
 
     param->pcap_dev = handle;
-    param->linkoffset = linkoffset;
     param->snaplen = snaplen;
     param->linktype = linktype;
 
@@ -381,7 +366,7 @@ int main(int argc, char *argv[]) {
             case 'b':
                 buff_size = atoi(optarg);
                 if (buff_size <= 0 || buff_size > 2047) {
-                    LogError("ERROR: Buffer size in MB must be betwee 0..2047 (2GB max)");
+                    LogError("ERROR: Buffer size in MB must be between 0..2047 (2GB max)");
                     exit(EXIT_FAILURE);
                 }
                 break;
@@ -403,7 +388,6 @@ int main(int argc, char *argv[]) {
                 } else {
                     port = "9995";
                 }
-                sendHost->family = AF_UNSPEC;
                 sendHost->hostname = strdup(optarg);
                 sendHost->port = strdup(port);
             } break;
@@ -524,7 +508,7 @@ int main(int argc, char *argv[]) {
                 if (verbose < 4) verbose++;
                 break;
             case 'V':
-                printf("%s: Version: %s", argv[0], nfdump_version);
+                printf("%s: %s\n", argv[0], versionString());
                 exit(EXIT_SUCCESS);
                 break;
             default:
@@ -578,8 +562,7 @@ int main(int argc, char *argv[]) {
             LogError("ERROR: Port to send flows is not a regular port.");
             exit(EXIT_FAILURE);
         }
-        sendHost->sockfd =
-            Unicast_send_socket(sendHost->hostname, sendHost->port, sendHost->family, bufflen, &(sendHost->addr), &(sendHost->addrlen));
+        sendHost->sockfd = Unicast_send_socket(sendHost->hostname, sendHost->port, AF_UNSPEC, bufflen, &(sendHost->addr), &(sendHost->addrlen));
         if (sendHost->sockfd <= 0) exit(EXIT_FAILURE);
         dbg_printf("Replay flows to host: %s port: %s\n", sendHost->hostname, sendHost->port);
         flowParam.sendHost = sendHost;
@@ -637,7 +620,7 @@ int main(int argc, char *argv[]) {
             exit(EXIT_FAILURE);
         }
 
-        if (InitBookkeeper(&fs->bookkeeper, fs->datadir, getpid(), launcher_pid) != BOOKKEEPER_OK) {
+        if (InitBookkeeper(&fs->bookkeeper, fs->datadir, getpid()) != BOOKKEEPER_OK) {
             LogError("initialize bookkeeper failed.");
             exit(EXIT_FAILURE);
         }
