@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2019-2022, Peter Haag
+ *  Copyright (c) 2019-2023, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -43,6 +43,7 @@
 
 #include "config.h"
 #include "content_dns.h"
+#include "exporter.h"
 #include "ifvrf.h"
 #include "ja3.h"
 #include "maxmind.h"
@@ -62,20 +63,43 @@ static void stringEXgenericFlow(FILE *stream, master_record_t *r) {
     char datestr1[64], datestr2[64], datestr3[64];
 
     struct tm *ts;
-    time_t when = r->msecFirst / 1000LL;
-    if (when == 0) {
-        strncpy(datestr1, "<unknown>", 63);
-    } else {
-        ts = localtime(&when);
-        strftime(datestr1, 63, "%Y-%m-%d %H:%M:%S", ts);
-    }
+    time_t when;
+
+    if (TestFlag(r->flags, V3_FLAG_EVENT)) {
+        uint64_t eventTime = r->msecFirst;
+#ifdef NSEL
+        if (r->msecEvent) eventTime = r->msecEvent;
+#endif
+        when = eventTime / 1000LL;
+        if (when == 0) {
+            strncpy(datestr1, "<unknown>", 63);
+        } else {
+            ts = localtime(&when);
+            strftime(datestr1, 63, "%Y-%m-%d %H:%M:%S", ts);
+        }
+        fprintf(stream, "  Event time   =     %13llu [%s.%03llu]\n", (long long unsigned)eventTime, datestr1, eventTime % 1000LL);
 
-    when = r->msecLast / 1000LL;
-    if (when == 0) {
-        strncpy(datestr2, "<unknown>", 63);
     } else {
-        ts = localtime(&when);
-        strftime(datestr2, 63, "%Y-%m-%d %H:%M:%S", ts);
+        when = r->msecFirst / 1000LL;
+        if (when == 0) {
+            strncpy(datestr1, "<unknown>", 63);
+        } else {
+            ts = localtime(&when);
+            strftime(datestr1, 63, "%Y-%m-%d %H:%M:%S", ts);
+        }
+
+        when = r->msecLast / 1000LL;
+        if (when == 0) {
+            strncpy(datestr2, "<unknown>", 63);
+        } else {
+            ts = localtime(&when);
+            strftime(datestr2, 63, "%Y-%m-%d %H:%M:%S", ts);
+        }
+
+        fprintf(stream,
+                "  first        =     %13llu [%s.%03llu]\n"
+                "  last         =     %13llu [%s.%03llu]\n",
+                (long long unsigned)r->msecFirst, datestr1, r->msecFirst % 1000LL, (long long unsigned)r->msecLast, datestr2, r->msecLast % 1000LL);
     }
 
     if (r->msecReceived) {
@@ -88,12 +112,9 @@ static void stringEXgenericFlow(FILE *stream, master_record_t *r) {
     }
 
     fprintf(stream,
-            "  first        =     %13llu [%s.%03llu]\n"
-            "  last         =     %13llu [%s.%03llu]\n"
             "  received at  =     %13llu [%s.%03llu]\n"
             "  proto        =               %3u %s\n"
             "  tcp flags    =              0x%.2x %s\n",
-            (long long unsigned)r->msecFirst, datestr1, r->msecFirst % 1000LL, (long long unsigned)r->msecLast, datestr2, r->msecLast % 1000LL,
             (long long unsigned)r->msecReceived, datestr3, (long long unsigned)r->msecReceived % 1000L, r->proto, ProtoString(r->proto, 0),
             r->proto == IPPROTO_TCP ? r->tcp_flags : 0, FlagsString(r->proto == IPPROTO_TCP ? r->tcp_flags : 0));
 
@@ -103,13 +124,14 @@ static void stringEXgenericFlow(FILE *stream, master_record_t *r) {
     }
 
     if (r->proto == IPPROTO_ICMP || r->proto == IPPROTO_ICMPV6) {  // ICMP
-        fprintf(stream, "  ICMP         =              %2u.%-2u type.code\n", r->icmp_type, r->icmp_code);
+        fprintf(stream, "  ICMP         =              %2u.%-2u type.code\n", r->icmpType, r->icmpCode);
     } else {
         fprintf(stream,
                 "  src port     =             %5u\n"
                 "  dst port     =             %5u\n"
-                "  src tos      =               %3u\n",
-                r->srcPort, r->dstPort, r->tos);
+                "  src tos      =               %3u\n"
+                "  fwd status   =               %3u\n",
+                r->srcPort, r->dstPort, r->tos, r->fwd_status);
     }
 
     fprintf(stream,
@@ -233,12 +255,11 @@ static void stringsEXflowMisc(FILE *stream, master_record_t *r) {
             "  output       =          %8u%s\n"
             "  src mask     =             %5u %s/%u\n"
             "  dst mask     =             %5u %s/%u\n"
-            "  fwd status   =               %3u\n"
             "  dst tos      =               %3u\n"
             "  direction    =               %3u\n"
             "  biFlow Dir   =              0x%.2x %s\n"
             "  end reason   =              0x%.2x %s\n",
-            r->input, ifInName, r->output, ifOutName, r->src_mask, snet, r->src_mask, r->dst_mask, dnet, r->dst_mask, r->fwd_status, r->tos, r->dir,
+            r->input, ifInName, r->output, ifOutName, r->src_mask, snet, r->src_mask, r->dst_mask, dnet, r->dst_mask, r->dst_tos, r->dir,
             r->biFlowDir, biFlowString(r->biFlowDir), r->flowEndReason, FlowEndString(r->flowEndReason));
 
 }  // End of stringsEXflowMisc
@@ -398,6 +419,30 @@ static void stringsEXlatency(FILE *stream, master_record_t *r) {
 
 }  // End of stringsEXlatency
 
+static void stringsEXsampler(FILE *stream, master_record_t *r) {
+    uint16_t exporterID = r->exporterSampler;
+
+    exporter_t *exporter = GetExporterInfo(exporterID);
+    if (exporter != NULL) {
+        sampler_t *sampler = exporter->sampler;
+        while (sampler) {
+            if (sampler->record.id == r->selectorID) break;
+            sampler = sampler->next;
+        }
+        if (sampler != NULL) {
+            fprintf(stream,
+                    "  samplingID   =             %5llu\n"
+                    "  pk Interval  =             %5u\n"
+                    "  sp Interval  =             %5u\n",
+                    (unsigned long long)r->selectorID, sampler->record.packetInterval, sampler->record.spaceInterval);
+        } else {
+            fprintf(stream, "  samplingID   =             %5llu\n", (unsigned long long)r->selectorID);
+        }
+    } else {
+        fprintf(stream, "  samplingID   =             %5llu\n", (unsigned long long)r->selectorID);
+    }
+}  // End of stringsEXsampler
+
 static void stringsEXobservation(FILE *stream, master_record_t *r) {
     fprintf(stream,
             "  obs domainID =         0x%05x\n"
@@ -497,21 +542,7 @@ static void stringsEXnselUserID(FILE *stream, master_record_t *r) {
 }  // End of stringsEXnselUserID
 
 static void stringsEXnelCommon(FILE *stream, master_record_t *r) {
-    char datestr[64];
-
-    time_t when = r->msecEvent / 1000LL;
-    if (when == 0) {
-        strncpy(datestr, "<unknown>", 63);
-    } else {
-        struct tm *ts = localtime(&when);
-        strftime(datestr, 63, "%Y-%m-%d %H:%M:%S", ts);
-    }
-
-    fprintf(stream,
-            "  nat event    =             %5u: %s\n"
-            "  Event time   =     %13llu [%s.%03llu]\n",
-            r->event, r->event_flag == FW_EVENT ? FwEventString(r->event) : EventString(r->event), (long long unsigned)r->msecEvent, datestr,
-            (long long unsigned)(r->msecEvent % 1000L));
+    fprintf(stream, "  nat event    =             %5u: %s\n", r->event, r->event_flag == FW_EVENT ? FwEventString(r->event) : EventString(r->event));
 
 }  // End of stringsEXnelCommon
 
@@ -615,23 +646,13 @@ static void inoutPayload(FILE *stream, master_record_t *r, char *payload, uint32
 }  // End of stringsEXoutPayload
 
 static void stringsEXpfinfo(FILE *stream, master_record_t *r) {
-    const char *pf_reasons[] = {"match",          "bad-offset",    "fragment",    "short",     "normalize",
-                                "memory)",        "bad-timestamp", "congestion",  "ip-option", "proto-cksum",
-                                "state-mismatch", "state-insert",  "state-limit", "src-limit", "synproxy"};
-
-    const char *reason;
-    if (r->pfReason <= 14)
-        reason = pf_reasons[r->pfReason];
-    else
-        reason = "<unknown>";
-
     fprintf(stream,
             "  pflog ifname =          %8s\n"
-            "  pflog action =             %5s\n"
-            "  pflog reason =             %5u/%s\n"
+            "  pflog action =             %5s/%u\n"
+            "  pflog reason =             %5s/%u\n"
             "  pflog direct =             %5s\n"
             "  pflog rulenr =             %5u\n",
-            r->pfIfName, r->pfAction ? "block" : "pass", r->pfReason, reason, r->pfDir ? "in" : "out", r->pfRulenr);
+            r->pfIfName, pfAction(r->pfAction), r->pfAction, pfReason(r->pfReason), r->pfReason, r->pfDir ? "in" : "out", r->pfRulenr);
 
 }  // End of stringsEXpfinfo
 
@@ -757,6 +778,9 @@ void raw_record(FILE *stream, void *record, int tag) {
             case EXlatencyID:
                 stringsEXlatency(stream, r);
                 break;
+            case EXsamplerInfoID:
+                stringsEXsampler(stream, r);
+                break;
             case EXobservationID:
                 stringsEXobservation(stream, r);
                 break;
