@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2022, Peter Haag
+ *  Copyright (c) 2009-2023, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -378,7 +378,7 @@ int main(int argc, char **argv) {
 
     if (!Init_nffile(NULL)) exit(254);
 
-    nffile = OpenNewFile("test.flows.nf", NULL, NOT_COMPRESSED, 0);
+    nffile = OpenNewFile("test.flows.nf", NULL, CREATOR_UNKNOWN, NOT_COMPRESSED, 0);
     if (!nffile) {
         exit(255);
     }
